# -*- coding: utf-8 -*-
"""
Created on Thu Jun 23 10:13:02 2016

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""

import numpy as np



###############################################################################
#::: Adjust scatter color according to airmass parabola
###############################################################################
#::: only 1 object in dic
def get_scatter_color(dic):
    print 'Fitting airmass polynomials.'
    dic['COLOR_PER_NIGHT'] = []
    
    for i, date in enumerate( dic['UNIQUE_NIGHT'] ):
        ind = np.where( dic['NIGHT'] == date )[0]

        t = dic['HJD'][ind]
        t = t - np.round(t[0]) #normalize to start of night; use "round" isntead of "int" to avoid shift by 1 day introduced by HJD offsets
        airm0 = dic['AIRMASS'][ind] - 1 #normalize to 0
        
        #::: color coding according to time/airmass
        #::: airmass ranges from 1 to 2
        c = airm0 
        p = np.polyfit( t, airm0, 2 ) #fit a 2D polynomial
        tmin = - p[1] / (2.*p[0]) #get the minimum of the polynomial
        c[ t < tmin ] *= - 1
        
        dic['COLOR_PER_NIGHT'].append(c)
        
    return dic  
    
    
    
#::: multiple objects in dic   
def get_scatter_color_multi(dic):
    print 'Fitting airmass polynomials.'
    dic['COLOR_PER_NIGHT'] = []
    
    for i, date in enumerate( dic['UNIQUE_NIGHT'] ):
        ind = np.where( dic['NIGHT'] == date )[0]

        t = dic['HJD'][0,ind]
        t = t - np.round(t[0]) #normalize to start of night; use "round" isntead of "int" to avoid shift by 1 day introduced by HJD offsets
        airm0 = dic['AIRMASS'][ind] - 1 #normalize to 0
        
        #::: color coding according to time/airmass
        #::: airmass ranges from 1 to 2
        c = airm0 
        p = np.polyfit( t, airm0, 2 ) #fit a 2D polynomial
        tmin = - p[1] / (2.*p[0]) #get the minimum of the polynomial
        c[ t < tmin ] *= - 1
        
        dic['COLOR_PER_NIGHT'].append(c)
        
    return dic  


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 15 13:54:17 2017

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""

###########################################################################
#::: import standard/community packages
########################################################################### 
import sys, timeit




NGTSIO_PATH = '/Users/mx/Data/Studium_aktuelles/PhD/NGTS/NGTSIO/'
if not NGTSIO_PATH in sys.path:
    sys.path.insert(0,NGTSIO_PATH)
    
CENTROIDING_PATH = '/Users/mx/Data/Studium_aktuelles/PhD/NGTS/CENTROIDING/centroiding_pip/'
if not CENTROIDING_PATH in sys.path:
    sys.path.insert(0,CENTROIDING_PATH)

print sys.path
print  '############################################################################'

#import ngtsio   
#import centd

#::: load pip centroiding
from ngtsio import ngtsio
from centroiding import centd




############################################################################
##::: MAIN
############################################################################             
if __name__ == '__main__':    
    
#    dic = ngtsio.get('NG0304-1115', ['PERIOD','EPOCH','WIDTH'], obj_id='012118', ngts_version='TEST18')
#    print dic
    fname = '/Users/mx/Big_Data/BIG_DATA_NGTS/2016/TEST18/NG0304-1115_809_2016_TEST18.fits'
    outdir = '/Users/mx/Data/Studium_aktuelles/PhD/NGTS/CENTROIDING/results/output/test/'
    centd.identify('NULL', '012118', 'NULL',
                   source='pipeline', fname_BLSPipe_megafile=fname,
                   outfname_info=outdir+'testpipe', outfname_plot=outdir+'testpipe',
                   user_period=66797.748228236189, user_width=12598.531879587919,
                   user_epoch=58118238.0)  
                   
                   
                   
    
    
    fname = '/Users/mx/Data/Studium_aktuelles/PhD/NGTS/CENTROIDING/results/input/candidate_shortlist_20171115.txt'
    outdir = '/Users/mx/Data/Studium_aktuelles/PhD/NGTS/CENTROIDING/results/output/test'
    centd.identify_list( fname, outdir=outdir )



############################################################################
##::: MAIN
############################################################################             
#if __name__ == '__main__':
#    
#    start = timeit.default_timer()
#    if len(sys.argv) == 1:
#        #::: TEST candidate
#        C = centroid('NG0304-1115', '019780', ngts_version='TEST18', user_period=69260.3241759, user_epoch=58102726.652, user_width=0.1*69260.3241759, dt=0.001)  
#        C.run()
#        
#    elif len(sys.argv) == 3:
#        print 'Input:', sys.argv[1], sys.argv[2]
#        C = centroid(str(sys.argv[1]), str(sys.argv[2]), show_plot=True)  
#        C.run()
#        
#    elif len(sys.argv) == 4:
#        print 'Input:', sys.argv[1], sys.argv[2], sys.argv[3]
#        C = centroid(str(sys.argv[1]), str(sys.argv[2]), ngts_version=str(sys.argv[3]) )  
#        C.run()
#    elif len(sys.argv) == 5:
#        print 'Input:', sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4]
#        C = centroid(str(sys.argv[1]), str(sys.argv[2]), ngts_version=str(sys.argv[3]), user_period=float(sys.argv[4]) )  
#        C.run()
#        
#    elif len(sys.argv) == 6:
#        print 'Input:', sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5]
#        C = centroid(str(sys.argv[1]), str(sys.argv[2]), ngts_version=str(sys.argv[3]), user_period=float(sys.argv[4]), user_epoch=float(sys.argv[5]) )
#        C.run()
#        
#    elif len(sys.argv) == 7:
#        print 'Input:', sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6]
#        C = centroid(str(sys.argv[1]), str(sys.argv[2]), ngts_version=str(sys.argv[3]), user_period=float(sys.argv[4]), user_epoch=float(sys.argv[5]), user_width=float(sys.argv[6]) )
#        C.run()        
#        
#    print 'execution time:', timeit.default_timer() - start


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 15 10:52:27 2017

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""


###########################################################################
#::: import standard/community packages
########################################################################### 
import os
import numpy as np
import matplotlib.pyplot as plt


###########################################################################
#::: import my custom packages
########################################################################### 
from mytools import lightcurve_tools


###########################################################################
#::: import sub-packages
########################################################################### 
#import stacked_images
import plotting



#def load_stacked_image(C):
#    '''
#    C              centroid class, passed over as 'self' from centroiding.py
#    '''
#    
#    refcatdir = 'RefCatPipe*' + C.fieldname + 
#    filename = glob.glob( os.path.join( C.root, 'DITHERED_STACK_'+fieldname+'*.fits' ) )[-1]
#    RefCatPipe_P.5046627_F.NG1318-4500_C.811_A.118433_T.REFCAT1706A
#    data = fitsio.read( C.stacked_image_filename )
    
    

###########################################################################
#::: plot
########################################################################### 
def plot(C):
    '''
    C      centroid class, passed over as 'self' from centroiding.py
    '''
    if C.dic_blends is not None:
        N = len(C.dic_blends['OBJ_ID']) + 1
        
        fig, axes = plt.subplots( N, 3, figsize=(15,N*4), sharey='col' )
        
        for i in range(N):
            j = i-1
            
            #::: plot CCD locations of target and all neighbours
            axes[i,0].plot( C.dic['CCDX_0'], C.dic['CCDY_0'], 'bo', ms=12 )
            axes[i,0].plot( C.dic_blends['CCDX_0'], C.dic_blends['CCDY_0'], 'k.' )
            if i==0:
                axes[i,0].plot( C.dic['CCDX_0'], C.dic['CCDY_0'], 'ro', ms=12 )            
            else:
                axes[i,0].plot( C.dic_blends['CCDX_0'][j], C.dic_blends['CCDY_0'][j], 'ro', ms=12 )
            axes[i,0].set( xlim=[ C.dic['CCDX'][0]-9, C.dic['CCDX'][0]+9 ], ylim=[ C.dic['CCDY'][0]-9, C.dic['CCDY'][0]+9 ],
                           xlabel='CCD x', ylabel='CCD y' )        
            circle1 = plt.Circle((C.dic['CCDX_0'], C.dic['CCDY_0'],), 3, color='r', fill=False, linewidth=3)
            axes[i,0].add_artist(circle1)
            axes[i,0].set_aspect('equal')
            
            #::: plot phase folded lightcurve of selected neighbour
            if i==0:
                hjd_phase, flux_phase = C.dic['HJD_PHASE'], C.dic['SYSREM_FLUX3_PHASE']
            else:
                hjd_phase, flux_phase, _, _, _ = lightcurve_tools.phase_fold(C.dic_blends['HJD'][j,:], C.dic_blends['SYSREM_FLUX3'][j,:], C.dic['PERIOD'], C.dic['EPOCH'], dt = C.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
                flux_phase /= np.nanmedian(flux_phase[ (hjd_phase<0.15) | ( (hjd_phase>0.35) & (hjd_phase<0.65) ) | (hjd_phase>0.85) ])
            #::: mask upwards outliers
            flux_phase[ flux_phase>1.1 ] = np.nan
            axes[i,1].plot( hjd_phase, flux_phase, 'bo', rasterized=True )
            axes[i,1].set( xlim=[-0.25,0.75], xlabel='Phase', ylabel='Flux' )
           
            #::: plot info page
            if i==0:
                plotting.plot_target_info_text(axes[i,2], C)
            else:
                plotting.plot_neighbour_info_text(axes[i,2], C, j)
            
        plt.tight_layout()
        
        
        
    else:
        fig, ax = plt.subplots( 1, 1, figsize=(15,4) )
        ax.set_xlim([0,1])
        ax.set_ylim([0,1])
        ax.axis('off')
        ax.text(0.5,0.5,'No blending objects resolved in NGTS',ha='center',va='center')
        
        
        
    outfilename = os.path.join( C.outdir, C.fieldname + '_' + C.obj_id + '_' + C.ngts_version + '_blends.pdf' )   
    plt.savefig( outfilename )
    
        
        
        

    


# -*- coding: utf-8 -*-
"""
Created on Tue Oct 25 14:57:36 2016

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""


###########################################################################
#::: import standard/community packages
########################################################################### 
import timeit, warnings
import numpy as np
import numpy.ma as ma
import matplotlib.pyplot as plt
#from scipy import signal
from astropy.stats import sigma_clip
from scipy.optimize import least_squares
import pandas as pd


###########################################################################
#::: import my custom packages
########################################################################### 
from mytools import lightcurve_tools, \
                    index_transits


###########################################################################
#::: import sub-packages
########################################################################### 
#pass







class detrend_centroid():
    
    def __init__(self, dic, dic_nb, method, R_min, N_top_max, dt, secondary_eclipse=True, t_exp=12., silent=True ):
        '''
        method: 'transit' or 'sidday': phase-fold and detrend on which period
        R_min: minimum Pearsons r for selection
        N
        '''
        
#        super(centroid, self).__init__(parent)
    
        self.dic = dic
        self.dic_nb = dic_nb
        self.method = method
        self.R_min = R_min #minimum Pearsons r correlation coefficient; to determine which neighbours will be included in the fit
        self.N_top_max = N_top_max #maximum number of neighbours for fit
        self.dt = dt
        self.secondary_eclipse = secondary_eclipse
        self.silent = silent #print output or not
        
        self.N_exp = len(self.dic['HJD'])
        self.N_neighbours = len(self.dic_nb['OBJ_ID'])
        if not self.silent: print 'N_neighbours:',self.N_neighbours
        
        self.t_exp = t_exp #12s exposure
        self.sidereal_day_period = 0.9972695787*24.*3600. #in s
        self.sidereal_day_epoch = int(dic['HJD'][0]/24./3600.)*24.*3600. #in s
        
        self.valid = True
#        self.dic['CENTDX'] += self.dic['CCDX']
#        self.dic['CENTDY'] += self.dic['CCDY']
#        self.dic_nb['CENTDX'] += self.dic_nb['CCDX']
#        self.dic_nb['CENTDY'] += self.dic_nb['CCDY']
        
#        self.run()
#        return self.dic, self.dic_nb  
    
    
    def run(self):
        '''
        1) Flatten the curves of the target and all neighbours per night
            -> use mean of each night (no sigma clipping)
        2) Find the most correlated neighbours
            -> select the 5 highest Pearson r, phase-folded on sidereal day period, ignoring in-transit data
        3) Detrend the target by the most correlated neighbours
            -> least squares fit, phase-folded on sidereal day period, ignoring in-transit data
        4) DO NOT remove remaining airmass trends from 1 sidereal day phasecurve
            -> second order polynomial
            
        _f : flattened (linear trend and offset removed per night) 
        _ref : reference curve computed from neighbouring stars
        _fd : flattened and detrended target star
        _fda: flattend, detrended, and airmass-detrended (1 sidereal day) target star
        '''
        
        #::: calculate breakpoints between different nights
        ind_bp, exposures_per_night, obstime_per_night = self.breakpoints()
        if not self.silent: print 'breakpoints done.'
        
        #::: get indices of in-transit and out-of-transit
        if self.secondary_eclipse is True:
            self.ind_out = index_transits.index_eclipses(self.dic)[-1]
        else:
            self.ind_out = index_transits.index_transits(self.dic)[3]
        if not self.silent: print 'index eclipses done.'
     
     
        
        #::: 0a)
        '''
        mask outliers 
        (all stars, overwrites keys 'CENTDXY' with masked arrays)
        '''
        
        start = timeit.default_timer()
        self.mask_outliers()
        self.mask_outliers_per_night()
        if not self.silent: print 'masking outliers done.'
        if not self.silent: print '\texecution time:', timeit.default_timer() - start, 's'
    
#        plt.figure(figsize=(200,6))
#        plt.plot( self.dic['HJD'], self.dic['CENTDX'], 'k.', rasterized=True)
#        self.mask_outliers()
#        plt.plot( self.dic['HJD'][ self.dic['CENTDX'].mask ], self.dic['CENTDX'].data[ self.dic['CENTDX'].mask ], 'ro', ms=15, rasterized=True)
#        plt.plot( self.dic['HJD'][ self.dic['CENTDX'].mask ], self.dic['CENTDX'].data[ self.dic['CENTDX'].mask ], 'bo', ms=12, rasterized=True)
##        self.mask_outliers_per_night_and_rolling()
##        plt.plot( self.dic['HJD'][ self.dic['CENTDX'].mask ], self.dic['CENTDX'].data[ self.dic['CENTDX'].mask ], 'co', ms=9, rasterized=True)                
#        plt.show()
        
        
        
        #::: 0b)
        '''
        mask the in-transit data for CENTDXY
        (target star only, creates new key 'CENTDXY_out'; out = out of transit)
        '''
        
        self.mask_transit(suffix='')
        if not self.silent: print 'masking transits in CENTDXY done.'
        
        
        #::: 0c)
        '''
        phase-fold all data on a sidereal day period, lunar period, and target period
        '''
        
        self.phase_fold_all_data(suffix='')
        if not self.silent: print 'phase-folding CENTDXY done.'
        
        
        
        
        #::: 1a) 
        '''
        remove night-to-night offsets
        (automatically only regards out-of-transit data for the target)
        '''
        
        self.flatten(suffix='_f')
        if not self.silent: print 'flattening (removing night-to-night offsets) done.'
        
        
        
        #::: 1b)
        '''
        mask the in-transit data for CENTDXY_f
        (target star only, creates new key 'CENTDXY_out_f'; out = out of transit)
        '''
        
        self.mask_transit(suffix='_f')
        if not self.silent: print 'masking transits in CENTDXY_f done.'
        
        
        
        #::: 1c)
        '''
        phase-fold all data on a sidereal day period, lunar period, and target period
        '''
        
        self.phase_fold_all_data(suffix='_f')
        if not self.silent: print 'phase-folding CENTDXY_f done.'
        
        
        
        #::: 2a)
        '''
        calculate correlation coefficients between the phase-folded centroids of target and all neighbours
        (on a sidereal day)
        (using the out-of-transit data only)
        '''
        
        #::: on a sidereal day
        if self.method == 'sidday':
            xkey_for_fit = 'CENTDX_out_f_PHASE_1sidday'
            ykey_for_fit = 'CENTDY_out_f_PHASE_1sidday'
            xkey_nb_for_fit = 'CENTDX_f_PHASE_1sidday'
            ykey_nb_for_fit = 'CENTDY_f_PHASE_1sidday'
        
        #::: on transit period
        if self.method == 'transit':
            xkey_for_fit = 'CENTDX_out_f_PHASE'
            ykey_for_fit = 'CENTDY_out_f_PHASE'
            xkey_nb_for_fit = 'CENTDX_f_PHASE'
            ykey_nb_for_fit = 'CENTDY_f_PHASE'
        
        self.dic_nb['corrcoeff_x'] = np.zeros(self.N_neighbours) * np.nan 
        self.dic_nb['corrcoeff_y'] = np.zeros(self.N_neighbours) * np.nan
        
#        print xkey_for_fit
#        print self.dic[xkey_for_fit]
#        print self.dic['CENTDX']
#        print self.dic['CENTDX_out']
#        print self.dic['CENTDX_f']
#        print self.dic['CENTDX_out_f']
#        print self.dic['CENTDX_f_PHASE']
#        print self.dic['CENTDX_out_f_PHASE']
        
#        err
        a_x = pd.Series( self.dic[xkey_for_fit] )
        a_y = pd.Series( self.dic[ykey_for_fit] )
        for i in range(self.dic_nb[xkey_nb_for_fit].shape[0]): 
            b_x = pd.Series( self.dic_nb[xkey_nb_for_fit][i] )
            b_y = pd.Series( self.dic_nb[ykey_nb_for_fit][i] )
#            print a_x
#            print b_x
#            print len(a_x)
#            print len(b_x)
            self.dic_nb['corrcoeff_x'][i] = a_x.corr( b_x )
#            print self.dic_nb['corrcoeff_x'][i]
            self.dic_nb['corrcoeff_y'][i] = a_y.corr( b_y )
        
        
        #::: choose stronlgy correlated neighbours for reference
        ind_x = np.where( self.dic_nb['corrcoeff_x'] > self.R_min )[0]
        ind_y = np.where( self.dic_nb['corrcoeff_y'] > self.R_min )[0]
        N_top_x = len( ind_x )
        N_top_y = len( ind_y )
#        print self.R_min
#        print self.dic_nb['corrcoeff_x']
#        print self.dic_nb['corrcoeff_y']
#        print 'N_top_x', N_top_x
#        print 'N_top_y', N_top_y
        
        #::: accept a maximum of N_top_max neighbouring objects for reference
        if N_top_x > self.N_top_max:
            N_top_x = self.N_top_max
            ind_x = np.argpartition(self.dic_nb['corrcoeff_x'], -N_top_x)[-N_top_x:]
            
        if N_top_y > self.N_top_max:
            N_top_y = self.N_top_max
            ind_y = np.argpartition(self.dic_nb['corrcoeff_y'], -N_top_y)[-N_top_y:]
#            print ind_y
#            print self.dic_nb['corrcoeff_y']
            
        if not self.silent: print 'N_top_x', N_top_x
        if not self.silent: print 'N_top_y', N_top_y
        
        if ( (N_top_x == 0) | (N_top_y == 0) ):
            warnings.warn('Not enough reference stars to perform fit!')
            warnings.warn('Reference set to NaN!')
            self.valid = False
        #pick the N_top highest corrcoeffs
#        ind_x = np.argpartition(self.dic_nb['corrcoeff_x'], -N_top)[-N_top:]
#        ind_y = np.argpartition(self.dic_nb['corrcoeff_y'], -N_top)[-N_top:]
        
        if not self.silent: print self.dic_nb['corrcoeff_x'][ind_x]
        if not self.silent: print self.dic_nb['corrcoeff_y'][ind_y]
        
        if not self.silent: print 'corrcoeff analysis done.'
        
    
        
        #::: 2b)
        '''
        find best fit of all neighbours
        (on a sidereal day)
        (using the out-of-transit data only)
        '''
        
        def errfct( params, centdxy_phase, xy_phase ):
            weights = params[:]
#            print xy_phase.shape
#            print weights.shape
            average, sum_of_weights = np.average(xy_phase, axis=0, weights=weights, returned=True)
            reference_curve = sum_of_weights * average
            return centdxy_phase - reference_curve
       
        def fit(N_top, key_for_fit, key_nb_for_fit, ind):
#            print key_for_fit, key_nb_for_fit
            #::: get parameters;  replace NaN bins with 0
            centdxy_phase = self.dic[key_for_fit]
            xy_phase = self.dic_nb[key_nb_for_fit][np.ix_( ind )]
            centdxy_phase[ ~np.isfinite(centdxy_phase) ] = 0.
            xy_phase[ ~np.isfinite(xy_phase) ] = 0.
            
            #::: perform least squars fit to find a local minimum
            p0 = np.ones( N_top ) / N_top #initial guess, one scale parameter plus N_top weight parameters; all initally have scale=1 and the same weight (=1/N_top)
            bounds = (np.zeros(N_top), np.zeros(N_top)+5. ) #bounds: [0,5]
            try:
                result = least_squares(errfct, p0, bounds=bounds, args=(centdxy_phase, xy_phase), loss='cauchy') 
            except:
                result = least_squares(errfct, p0, bounds=bounds, args=(centdxy_phase, xy_phase), loss='soft_l1') 
            
            #::: or perform a DE fit
#            bounds = zip( np.zeros(N_top)-5., np.zeros(N_top)+5. )
#            result = differential_evolution(errfct, bounds=bounds, args=(centdxy_phase, xy_phase)) 
            
            weights = result.x #'.x' calls the parameters of the fit, ie. here the weights
            return weights
        
        
        if self.valid:
            weights_x = fit(N_top_x, xkey_for_fit, xkey_nb_for_fit, ind_x)
            weights_y = fit(N_top_y, ykey_for_fit, ykey_nb_for_fit, ind_y)
        else:
            weights_x = np.nan
            weights_y = np.nan
        
        #::: save in dic_nb
        self.dic_nb['weights_x'] = np.zeros_like(self.dic_nb['corrcoeff_x']) 
        self.dic_nb['weights_x'][ ind_x ] = weights_x
        self.dic_nb['weights_y'] = np.zeros_like(self.dic_nb['corrcoeff_y']) 
        self.dic_nb['weights_y'][ ind_y ] = weights_y
        
        
        if not self.silent: 
            print 'x sum:', np.sum(weights_x)
            print 'x result:', np.sum(weights_x) * weights_x 
            print 'y sum:', np.sum(weights_y)
            print 'y result:', np.sum(weights_y) * weights_y
        
        
        
        #::: 2c)
        '''
        add info into the dictionaries (non-phase-folded)
        (now for all data again)
        '''        
        
        if self.valid:
            self.dic_nb['CENTDX_ref_mean'] = np.sum(weights_x) * np.average(self.dic_nb['CENTDX_f'][ ind_x, : ], axis=0, weights=weights_x)
            self.dic_nb['CENTDY_ref_mean'] = np.sum(weights_y) * np.average(self.dic_nb['CENTDY_f'][ ind_y, : ], axis=0, weights=weights_y)
        else:
            self.dic_nb['CENTDX_ref_mean'] = np.zeros_like(self.dic['CENTDX_f'])*np.nan
            self.dic_nb['CENTDY_ref_mean'] = np.zeros_like(self.dic['CENTDY_f'])*np.nan
            
        self.dic['CENTDX_out_fd'] = self.dic['CENTDX_out_f'] - self.dic_nb['CENTDX_ref_mean']
        self.dic['CENTDY_out_fd'] = self.dic['CENTDY_out_f'] - self.dic_nb['CENTDY_ref_mean']
        self.dic['CENTDX_fd'] = self.dic['CENTDX_f'] - self.dic_nb['CENTDX_ref_mean']
        self.dic['CENTDY_fd'] = self.dic['CENTDY_f'] - self.dic_nb['CENTDY_ref_mean']

    

        #::: 2d)
        '''
        mask the in-transit data for CENTDXY_fd
        (target star only, creates new key 'CENTDXY_out_fd'; out = out of transit)
        '''
        
        self.mask_transit(suffix='_fd')
        if not self.silent: print 'masking transits in CENTDXY_fd done.'
        
        
        
        #::: 2e)
        '''
        flatten again per night
        '''
        self.flatten(suffix='_fd')
        if not self.silent: print 'flattening 2 (removing night-to-night offsets) done.'
    
    
        
        #::: 2f)
        '''
        phase-fold all data on a sidereal day period, lunar period, and target period
        '''
        
        self.phase_fold_all_data(suffix='_fd')
        self.phase_fold_all_data(suffix='_ref_mean')
        if not self.silent:print 'phase-folding CENTDXY_fd and CENTDXY_ref_mean done.'
        
                  
    
        #::: 3a)
        '''
        creates new key 'CENTDXY_fda'
        '''
              
        self.fit_phasecurve_1_sidereal_day()
        if not self.silent:print 'fitted sidereal day correction; created new key CENTDXY_fda'



        #::: 3b)
        '''
        mask the in-transit data for CENTDXY_fda
        (target star only, creates new key 'CENTDXY_out_fda'; out = out of transit)
        '''
        
        self.mask_transit(suffix='_fda')
        if not self.silent:print 'masking transits in CENTDXY_fda done; created new key CENTDXY_out_fda'
        
        
        #::: 3c)
        '''
        flatten again per night
        '''
        self.flatten(suffix='_fda')
        if not self.silent: print 'flattening 3 (removing night-to-night offsets) done.'
    
        
        
        #::: 3d)
        '''
        phase-fold all data on a sidereal day period, lunar period, and target period
        '''
        
        self.phase_fold_all_data(suffix='_fda')
        if not self.silent:print 'phase-folding CENTDXY_fda done.'
          
            
            
        #::: return    
        return self.dic, self.dic_nb
        
        
        

    def mask_outliers(self, sigma=5):
        self.dic['CENTDX'] = sigma_clip(self.dic['CENTDX'], sigma=sigma, iters=1)
        self.dic['CENTDY'] = sigma_clip(self.dic['CENTDY'], sigma=sigma, iters=1)
        self.dic_nb['CENTDX'] = sigma_clip(self.dic_nb['CENTDX'], sigma=sigma, iters=1, axis=1) #astropy's sigma_clip does work for matrixes per row/column!
        self.dic_nb['CENTDY'] = sigma_clip(self.dic_nb['CENTDY'], sigma=sigma, iters=1, axis=1)
            
            
            
            
    def mask_outliers_per_night(self, sigma=4):
        for i, date in enumerate(self.dic['UNIQUE_NIGHT']): 
            
            #::: select one night + select out of transit
            ind_date = np.where( self.dic['NIGHT'] == date )[0]
#            ind_date_out = np.intersect1d( ind_date, self.ind_out )
            
            #::: catch 'empty' days (where all values have already been flagged / set to NAN / masked)
            def try_sigma_clip(x, sigma, iters, axis=None):
                x = np.ma.masked_invalid(x)
                if ( x.mask.all() ):
                    return x
                else:
                    return sigma_clip(x, sigma=sigma, iters=iters, axis=axis)
                    
            #::: 
            self.dic['CENTDX'][ind_date] = try_sigma_clip(self.dic['CENTDX'][ind_date], sigma=sigma, iters=1)
            self.dic['CENTDY'][ind_date] = try_sigma_clip(self.dic['CENTDY'][ind_date], sigma=sigma, iters=1)
            self.dic_nb['CENTDX'][:,ind_date] = try_sigma_clip(self.dic_nb['CENTDX'][:,ind_date], sigma=sigma, iters=1, axis=1) #astropy's sigma_clip does work for matrixes per row/column!
            self.dic_nb['CENTDY'][:,ind_date] = try_sigma_clip(self.dic_nb['CENTDY'][:,ind_date], sigma=sigma, iters=1, axis=1)
            
      
      
            
    def mask_outliers_per_night_and_rolling(self, sigma=4):

        window = 600./self.t_exp        
        lag_step = window/5.
        
        for i, date in enumerate(self.dic['UNIQUE_NIGHT']): 
            
            #::: select one night + select out of transit
            ind_date = np.where( self.dic['NIGHT'] == date )[0]
#            ind_date_out = np.intersect1d( ind_date, self.ind_out )
            
            #::: go through it in 10 minute steps (not quite rolling, but kinda...) 
            #::: -> more computation time efficient than rolling, and no border-effects
            #::: pick out the data points in intervals
            #            0:50
            #            50:100
            #            100:150 etc
            j0 = 0
            while j0 < len(ind_date):
                j1 = np.min( (len(ind_date), j0+window) )
#                print j0
#                print j1
#                print type(j0)
#                print type(j1)
                ind_select = ind_date[ int(j0):int(j1) ]
                
                #::: 
                try:
                    self.dic['CENTDX'][ind_select] = sigma_clip(self.dic['CENTDX'][ind_select], sigma=sigma, iters=1)
                    self.dic['CENTDY'][ind_select] = sigma_clip(self.dic['CENTDY'][ind_select], sigma=sigma, iters=1)
                    self.dic_nb['CENTDX'][:,ind_select] = sigma_clip(self.dic_nb['CENTDX'][:,ind_select], sigma=sigma, iters=1, axis=1) #astropy's sigma_clip does work for matrixes per row/column!
                    self.dic_nb['CENTDY'][:,ind_select] = sigma_clip(self.dic_nb['CENTDY'][:,ind_select], sigma=sigma, iters=1, axis=1)
                except:
                    print 'sigma clip failure for', date, j0, j1
                
#                j0 += N_pointstep
                j0 += lag_step

            
            
#    def mask_outliers_RM(self, x):
#        x = pd.Series(1.*x)
#        
#        threshold = 3
#        window = 50 #10mins a 12s exposure = 600s/12s = 50 data points
#        buf = pd.rolling_median(x, window=window, center=True).fillna(method='bfill').fillna(method='ffill')
#        difference = np.abs(x - buf)
#        outlier_ind = difference > threshold*np.std(x)
#        x[ outlier_ind ] = np.nan
#        
#        return x
    
        
        
    def mask_transit(self, suffix):
        mask = np.zeros(self.N_exp, dtype=int)
#        mask[self.ind_tr] = 1 #mask the in-transit data
        self.dic['CENTDX_out'+suffix] = ma.masked_array(self.dic['CENTDX'+suffix], mask=mask, copy=True)
        self.dic['CENTDY_out'+suffix] = ma.masked_array(self.dic['CENTDY'+suffix], mask=mask, copy=True)
        
        
    
    def breakpoints(self):
        '''
        Mark breakpoints (start and end of nights)
        '''
        ind_bp = [0]
        exposures_per_night = []
        obstime_per_night = []
        for i, date in enumerate(self.dic['UNIQUE_NIGHT']):    
            #::: get the exposures of this night
            ind = np.where( self.dic['NIGHT'] == date )[0] 
            exposures_per_night.append( len(ind) )
            obstime_per_night.append( self.dic['HJD'][ind[-1]] - self.dic['HJD'][ind[0]] )
    #        ind_bp.append(ind[0])
            ind_bp.append(ind[-1])
        return ind_bp, np.array(exposures_per_night), np.array(obstime_per_night)
    
        
        
     
    def phase_fold_all_data(self, suffix=''):
        
        def pf( y, mode ):
            if mode=='transit':  return lightcurve_tools.phase_fold(self.dic['HJD'], y, self.dic['PERIOD'], self.dic['EPOCH'],             dt=self.dt, ferr_type='meansig', ferr_style='std', sigmaclip=False ) 
            elif mode=='sidday': return lightcurve_tools.phase_fold(self.dic['HJD'], y, self.sidereal_day_period, self.sidereal_day_epoch, dt=self.dt, ferr_type='meansig', ferr_style='std', sigmaclip=False ) 

        def pf_matrix( y, mode ):
            if mode=='transit':  return lightcurve_tools.phase_fold_matrix(self.dic['HJD'], y, self.dic['PERIOD'], self.dic['EPOCH'],             dt=self.dt, ferr_type='meansig', ferr_style='std', sigmaclip=False ) 
            elif mode=='sidday': return lightcurve_tools.phase_fold_matrix(self.dic['HJD'], y, self.sidereal_day_period, self.sidereal_day_epoch, dt=self.dt, ferr_type='meansig', ferr_style='std', sigmaclip=False ) 

                
        if suffix in ['', '_f', '_fd', '_fda']:
            
            _, self.dic['CENTDX'+suffix+'_PHASE'], _, _, _ = pf( self.dic['CENTDX'+suffix], 'transit' )
            _, self.dic['CENTDY'+suffix+'_PHASE'], _, _, _ = pf( self.dic['CENTDY'+suffix], 'transit' )
            _, self.dic['CENTDX'+suffix+'_PHASE_1sidday'], _, _, _ = pf( self.dic['CENTDX'+suffix], 'sidday' )
            _, self.dic['CENTDY'+suffix+'_PHASE_1sidday'], _, _, _ = pf( self.dic['CENTDY'+suffix], 'sidday' )  
            
            _, self.dic['CENTDX_out'+suffix+'_PHASE'], _, _, _ = pf( self.dic['CENTDX_out'+suffix], 'transit' )
            _, self.dic['CENTDY_out'+suffix+'_PHASE'], _, _, _ = pf( self.dic['CENTDY_out'+suffix], 'transit' )          
            _, self.dic['CENTDX_out'+suffix+'_PHASE_1sidday'], _, _, _ = pf( self.dic['CENTDX_out'+suffix], 'sidday' )
            _, self.dic['CENTDY_out'+suffix+'_PHASE_1sidday'], _, _, _ = pf( self.dic['CENTDY_out'+suffix], 'sidday' )
           
        if suffix in ['_f']:
            
            _, self.dic_nb['CENTDX'+suffix+'_PHASE'], _, _, _ = pf_matrix( self.dic_nb['CENTDX'+suffix], 'transit' ) 
            _, self.dic_nb['CENTDY'+suffix+'_PHASE'], _, _, _ = pf_matrix( self.dic_nb['CENTDY'+suffix], 'transit' )    
            _, self.dic_nb['CENTDX'+suffix+'_PHASE_1sidday'], _, _, _ = pf_matrix( self.dic_nb['CENTDX'+suffix], 'sidday' )    
            _, self.dic_nb['CENTDY'+suffix+'_PHASE_1sidday'], _, _, _ = pf_matrix( self.dic_nb['CENTDY'+suffix], 'sidday' )
           
        if suffix in ['_ref_mean']:
            
            _, self.dic_nb['CENTDX'+suffix+'_PHASE'], _, _, _ = pf( self.dic_nb['CENTDX'+suffix], 'transit' ) 
            _, self.dic_nb['CENTDY'+suffix+'_PHASE'], _, _, _ = pf( self.dic_nb['CENTDY'+suffix], 'transit' )    
            _, self.dic_nb['CENTDX'+suffix+'_PHASE_1sidday'], _, _, _ = pf( self.dic_nb['CENTDX'+suffix], 'sidday' )    
            _, self.dic_nb['CENTDY'+suffix+'_PHASE_1sidday'], _, _, _ = pf( self.dic_nb['CENTDY'+suffix], 'sidday' )
           
           
    
    
    def flatten(self, suffix='_f'):    
        
        #::: create the dictionary keys if needed
        if suffix == '_f':
            self.dic['CENTDX'+suffix] = self.dic['CENTDX']*1.
            self.dic['CENTDY'+suffix] = self.dic['CENTDY']*1.
            self.dic_nb['CENTDX'+suffix] = self.dic_nb['CENTDX']*1.
            self.dic_nb['CENTDY'+suffix] = self.dic_nb['CENTDY']*1.
        
        #::: then go through night-by-night
        for i, date in enumerate(self.dic['UNIQUE_NIGHT']): 
            
            #::: select one night + select out of transit
            ind_date = np.where( self.dic['NIGHT'] == date )[0]
            ind_date_out = np.intersect1d( ind_date, self.ind_out )
            
            #::: subtract nightly offsets
            self.dic['CENTDX'+suffix][ ind_date ] = self.dic['CENTDX'+suffix][ ind_date ] - np.nanmean( self.dic['CENTDX'+suffix][ ind_date_out ] )
            self.dic['CENTDY'+suffix][ ind_date ] = self.dic['CENTDY'+suffix][ ind_date ] - np.nanmean( self.dic['CENTDY'+suffix][ ind_date_out ] )
            try:
                self.dic_nb['CENTDX'+suffix][ :, ind_date ] = self.dic_nb['CENTDX'+suffix][ :, ind_date ] - np.nanmean( self.dic_nb['CENTDX'+suffix][ :, ind_date ], axis=1 )[:, np.newaxis]
                self.dic_nb['CENTDY'+suffix][ :, ind_date ] = self.dic_nb['CENTDY'+suffix][ :, ind_date ] - np.nanmean( self.dic_nb['CENTDY'+suffix][ :, ind_date ], axis=1 )[:, np.newaxis]
            except:
                pass
            
#        fig, axes = plt.subplots(2,1,figsize=(80,6))
#
##        axes[0].plot( self.dic['HJD'], self.dic['CENTDX'], 'g.', rasterized=True  )
#        axes[0].plot( self.dic['HJD'], self.dic['CENTDX_f'], 'b.', rasterized=True  )
##        axes[0].plot( self.dic['HJD'][self.ind_tr], self.dic['CENTDX_f'][self.ind_tr], 'r.', rasterized=True )
#        
##        axes[1].plot( self.dic['HJD'], self.dic_nb['CENTDX'][1, :], 'g.', rasterized=True  )            
#        axes[1].plot( self.dic['HJD'], self.dic_nb['CENTDX_f'][0, :], 'b.', rasterized=True  )
#            
#        err
        
        
        
#    def detrend_per_night(dic, data, ind_out):    
#        #::: copy data array    
#        data_detrended = 1.*data
#        
#        for i, date in enumerate(dic['UNIQUE_NIGHT']): 
#            
#            #TODO: this sometimes breaks with error message
#            #          File "/appch/data1/mg719/programs/anaconda/lib/python2.7/site-packages/astropy/stats/sigma_clipping.py", line 208, in _sigma_clip
#            #    perform_clip(filtered_data, kwargs)
#            #  File "/appch/data1/mg719/programs/anaconda/lib/python2.7/site-packages/astropy/stats/sigma_clipping.py", line 180, in perform_clip
#            #    _filtered_data.mask |= _filtered_data < min_value
#            #TypeError: ufunc 'bitwise_or' not supported for the input types, and the inputs could not be safely coerced to any supported types according to the casting rule ''safe''
#            try:
#                #::: select one night + select out of transit
#                ind_date = np.where( dic['NIGHT'] == date )[0]
#                ind = np.intersect1d( ind_date, ind_out )
#                
#                #::: calculate sigma clipped mean (using astropy)
#                offset = np.mean( sigma_clip(data[ ind ]) )
#                
#                data_detrended[ ind_date ] -= offset     
#            
#            except:
#                pass
#            
#        return data_detrended
#        
#    
#    
#    def detrend_global(dic, data, ind_out):    
#        #::: copy data array    
#        data_detrended = 1.*data
#        
#        #::: select out of transit
#        ind = ind_out
#        
#        #::: calculate sigma clipped mean (using astropy)
#        offset = np.mean( sigma_clip(data[ ind ]) )
#        
#        data_detrended -= offset     
#            
#        return data_detrended   
        
        
     
    def fit_phasecurve_1_sidereal_day(self, polyorder=None):
        '''
        use out-of-transit data only
        
        1 mean sidereal day = 
        ( 23 + 56/60. + 4.0916/3600. ) / 24. = 0.9972695787 days
        '''
        
        #::: phasefold to one sidereal day
#        self.dic['HJD_PHASE_1sidday'], self.dic['COLOR_PHASE_1sidday'], self.dic['COLOR_PHASE_1sidday_ERR'], _, _ = lightcurve_tools.phase_fold( self.dic['HJD'], self.dic['COLOR'], self.sidereal_day_period, self.sidereal_day_epoch, dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
#        _, self.dic['AIRMASS_PHASE_1sidday'], self.dic['AIRMASS_PHASE_1sidday_ERR'], _, _ = lightcurve_tools.phase_fold( self.dic['HJD'], self.dic['AIRMASS'], self.sidereal_day_period, self.sidereal_day_epoch, dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
        _, self.dic['SYSREM_FLUX3_PHASE_1sidday'], self.dic['SYSREM_FLUX3_PHASE_1sidday_ERR'], _, _ = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['SYSREM_FLUX3'] / np.nanmedian(self.dic['SYSREM_FLUX3']), self.sidereal_day_period, self.sidereal_day_epoch, dt = self.dt, ferr_type='meansig', ferr_style='std', sigmaclip=True)
        
        #::: for CENTDXY use out of transit data only        
        self.dic['HJD_PHASE_1sidday'], self.dic['CENTDX_out_fd_PHASE_1sidday'], self.dic['CENTDX_out_fd_PHASE_1sidday_ERR'], _, _ = lightcurve_tools.phase_fold( self.dic['HJD'], self.dic['CENTDX_out_fd'], self.sidereal_day_period, self.sidereal_day_epoch, dt = self.dt, ferr_type='meansig', ferr_style='std', sigmaclip=True)
        _,                             self.dic['CENTDY_out_fd_PHASE_1sidday'], self.dic['CENTDY_out_fd_PHASE_1sidday_ERR'], _, _ = lightcurve_tools.phase_fold( self.dic['HJD'], self.dic['CENTDY_out_fd'], self.sidereal_day_period, self.sidereal_day_epoch, dt = self.dt, ferr_type='meansig', ferr_style='std', sigmaclip=True)
        
        
        
        #::: detrend via polyfit
        if polyorder is not None:
            
            def polyfit(x, y):
    #            ind_infinite = ~ np.isfinite(y)
    #            x[ ind_infinite ] = 0
    #            y[ ind_infinite ] = 0
                ind_finite = np.isfinite(x) & np.isfinite(y)
                x = x[ ind_finite ]
                y = y[ ind_finite ]
                return np.polyfit( x, y, polyorder )
                
            #::: fit out the trend and save the resulting polyfunction in the self.dic
            polyfit_params = polyfit( self.dic['HJD_PHASE_1sidday'], self.dic['CENTDX_out_fd_PHASE_1sidday'] )
            self.dic['polyfct_CENTDX'] = np.poly1d( polyfit_params )
                
            polyfit_params = polyfit( self.dic['HJD_PHASE_1sidday'], self.dic['CENTDY_out_fd_PHASE_1sidday'] )
            self.dic['polyfct_CENTDY'] = np.poly1d( polyfit_params )
        
            #::: unwrap the phase-folding
            dx = ( (self.dic['HJD'] - self.sidereal_day_epoch) % (self.sidereal_day_period) ) / self.sidereal_day_period 
            self.dic['poly_CENTDX'] = self.dic['polyfct_CENTDX'](dx)
            self.dic['poly_CENTDY'] = self.dic['polyfct_CENTDY'](dx)
        
            self.dic['CENTDX_fda'] = self.dic['CENTDX_fd'] - self.dic['poly_CENTDX']
            self.dic['CENTDY_fda'] = self.dic['CENTDY_fd'] - self.dic['poly_CENTDY']
            self.dic['CENTDX_out_fda'] = self.dic['CENTDX_out_fd'] - self.dic['poly_CENTDX']
            self.dic['CENTDY_out_fda'] = self.dic['CENTDY_out_fd'] - self.dic['poly_CENTDY']
        
        
        #::: detrend via moving average (median) fit 
        #TODO: currently this removes data points on the edges (not covered in the rolling window) by setting them NAN; try to extrapolate those?
        else:
        
            def ma_fit(y):
                rolling_mean = pd.Series(y).rolling(window=int(len(y)/50.), center=True).mean()
#                rolling_mean[ np.isnan(rolling_mean) ] = 0.
                return rolling_mean.as_matrix()
                
            def ma_fct(t, tp, yp):
                #::: if the phase-folding is not starting at phase 0 (but e.g. at 0.25),
                #::: offset it to phase 0
                if (tp[0] < 0.):
                    ind = np.where( tp<0. )[0]
                    tp = np.append( tp[ind[-1]+1:], tp[ind]+1. )
                    yp = np.append( yp[ind[-1]+1:], yp[ind] )
                return np.interp(t, tp, yp)
                
            #::: fit out the trend and save the resulting polyfunction in the self.dic
            self.dic['ma_CENTDX_PHASE_1sidday'] = ma_fit( self.dic['CENTDX_out_fd_PHASE_1sidday'] )
            self.dic['ma_CENTDY_PHASE_1sidday'] = ma_fit( self.dic['CENTDY_out_fd_PHASE_1sidday'] )
        
            #::: unwrap the phase-folding
            dx = ( (self.dic['HJD'] - self.sidereal_day_epoch) % (self.sidereal_day_period) ) / self.sidereal_day_period 
            self.dic['ma_CENTDX'] = ma_fct( dx, self.dic['HJD_PHASE_1sidday'], self.dic['ma_CENTDX_PHASE_1sidday'] )
            self.dic['ma_CENTDY'] = ma_fct( dx, self.dic['HJD_PHASE_1sidday'], self.dic['ma_CENTDY_PHASE_1sidday'] )
        
        
            self.dic['CENTDX_fda'] = self.dic['CENTDX_fd'] - self.dic['ma_CENTDX']
            self.dic['CENTDY_fda'] = self.dic['CENTDY_fd'] - self.dic['ma_CENTDY']
            self.dic['CENTDX_out_fda'] = self.dic['CENTDX_out_fd'] - self.dic['ma_CENTDX']
            self.dic['CENTDY_out_fda'] = self.dic['CENTDY_out_fd'] - self.dic['ma_CENTDY']

#        _, self.dic['CENTDX_fda_PHASE_1sidday'], self.dic['CENTDX_fda_PHASE_1sidday_ERR'], _, _ = lightcurve_tools.phase_fold( self.dic['HJD'], self.dic['CENTDX_fda'], self.sidereal_day_period, t0, dt = self.dt, ferr_type='meansig', ferr_style='std', sigmaclip=True)
#        _, self.dic['CENTDY_fda_PHASE_1sidday'], self.dic['CENTDY_fda_PHASE_1sidday_ERR'], _, _ = lightcurve_tools.phase_fold( self.dic['HJD'], self.dic['CENTDY_fda'], self.sidereal_day_period, t0, dt = self.dt, ferr_type='meansig', ferr_style='std', sigmaclip=True)
        
        
#        print self.dic['poly_CENTDX']
#        plt.figure(figsize=(12,6))
#        plt.plot( self.dic['HJD'], self.dic['CENTDX_fd_PHASE'], 'g.', rasterized=True )
#        plt.plot( self.dic['HJD'], self.dic['CENTDX_fda_PHASE'], 'b.', rasterized=True )
#        plt.title('target period')
#        plt.show()
#        
#        plt.figure(figsize=(12,6))
#        plt.plot( self.dic['HJD'], self.dic['CENTDX_fd_PHASE_1sidday'], 'g.', rasterized=True )
#        plt.plot( self.dic['HJD'], self.dic['CENTDX_fda_PHASE_1sidday'], 'b.', rasterized=True )
#        plt.title('1 sid day')
#        plt.show()
        
#        dxx = np.arange( self.dic['HJD'][0], self.dic['HJD'][-1], 600 )
#        dxx_wrapped = ( (dxx - t0) % (self.sidereal_day_period) ) / self.sidereal_day_period 
#        plt.plot( dxx, self.dic['polyfct_CENTDX'](dxx_wrapped)*100, 'r-', rasterized=True )
#        plt.show()
        
        
       
    def run_example(self):
        '''
        For test purposes
        '''
        #::: choose example data
        data = self.dic['CENTDX']
        time = self.dic['HJD']
        
        ind_bp = self.breakpoints()
#        data_detrended = detrend_scipy(data, ind_bp)
#        plot(data, data_detrended, time)    
        
        
        
    def plot(data, data_detrended, time): 
        '''
        For test purposes
        '''   
        fig, axes = plt.subplots(2,1, sharex=True, figsize=(12,4))
        N_exp = 1e4
        
        ax = axes[0]
        ax.scatter( np.arange(N_exp), data[0:N_exp], c=time[0:N_exp].astype(int), rasterized=True, cmap='jet' )
        
        ax = axes[1]
        ax.scatter( np.arange(N_exp), data_detrended[0:N_exp], c=time[0:N_exp].astype(int), rasterized=True, cmap='jet' )
    
        ax.set( xlim=[0,N_exp] )        
    



# -*- coding: utf-8 -*-
"""
Created on Sun Aug 14 18:25:52 2016

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""


###########################################################################
#::: import standard/community packages
########################################################################### 
import os, glob, warnings
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import matplotlib.gridspec as gridspec
from matplotlib.backends.backend_pdf import PdfPages
import pandas as pd
from scipy.stats import binom_test, ttest_1samp
from astropy.io import fits
#import statsmodels.api as sm



###########################################################################
#::: import my custom packages
########################################################################### 
from ngtsio import ngtsio
from mytools import index_transits, \
                    lightcurve_tools, \
                    pandas_tsa, \
                    set_nan
from mytools.utils import mystr 



###########################################################################
#::: import sub-packages
########################################################################### 
from scripts import detrend_centroid_external, \
                    analyse_neighbours, \
                    get_scatter_color, \
                    inspect_blends, \
                    plotting
#                    stacked_images


###########################################################################
#::: plot settings
########################################################################### 
try:
    import seaborn as sns
    sns.set(context='paper', style='ticks', palette='deep', font='sans-serif', font_scale=1.5, color_codes=True)
    sns.set_style({"xtick.direction": "in","ytick.direction": "in"})
except ImportError:
    pass


import matplotlib.pylab as pylab
params = {'legend.fontsize': 'medium',
         'axes.labelsize': 'medium',
         'axes.titlesize':'medium',
         'xtick.labelsize':'medium',
         'ytick.labelsize':'medium',
         'font.size': 18.}
pylab.rcParams.update(params)

# Turn interactive plotting off
plt.ioff()

#import matplotlib
#::: MNRAS STYLE 
#matplotlib.rc('font',**{'family':'serif', 
#             'serif':['Times'], 
#             'weight':'normal', 
#             'size':18})
#matplotlib.rc('legend',**{'fontsize':18})
#matplotlib.rc('text', usetex=True)




###########################################################################
#::: centroid class
########################################################################### 
class centroid():
    '''
    Parameters
    ----------
    fieldname : str
        NGTS fieldname 
    obj_id : int / string
        NGTS object ID
    ngts_version : str
        NGTS pipeline version
    source : str 
        BLS or CANVAS or pipeline
    bls_rank : int
        BLS rank, 1 to 5
    user_period : float
        user-given transit period; in s
    user_epoch : float
        user-given transit epoch; in s
    user_width : float
        user-given transit duration; in s
    user_flux : float
        user-given flux time series (to avoid loading it via ngtsio)
    user_centdx : float
        user-given centdx time series (to avoid loading it via ngtsio)
    user_centdy : float 
        user-given centdy time series (to avoid loading it via ngtsio)
    time_hjd : float
        only use the time series at the given hjd 
    pixel_radius : float
        radius in which to search for reference objects
    flux_min : float
        minimum flux of reference object
    flux_max : float
        maximum flux of reference object
    method : str
        phase-fold on transit period or sidereal day for correlation analysis and detrending
    R_min : float
        minimum Pearson's R required to select a reference object
    N_top_max : int
        maximum number of reference objects
    bin_width : float
        300; in seconds; bin_width of the full light curve
    min_time : float
        -
    dt : float
        0.005; in phase-units; bin_width in the phase-folded analysis
    secondary_eclipse : bool
        mask out a possible secondary eclipse when correlating
    blend_radius : float
        for an aperture of 3 pixels and PSF of ~1 pixel, we assume
        only objects <=6 pixels away will blend into the photometric aperture of the target
    roots : dict
        dictionary of file roots for use with ngtsio.py
    outdir : str
        -
    parent : -
        -
    do_plot : str
        'none', 'minimal', 'all'
    show_plot : bool
        whether to show plots in python;
        if False, they still get saved
    flagfile : str
        filename of a flagfile to write output per object, used for centroiding_multi.py
    dic : dict
        dictionary of all necessary data, if None it's read in with ngtsio here
    nancut : array-like
        list of True/False for which photometric time eries entries will be ignored;
        needed only for autovetter.py
    output : str
        'minimal', 'all', 'pipeline'
    outfname_info: str
        for in-pipeline use; fname of the infofile if user set
    outfname_plot: str
        for in-pipeline use; fname of the plotfile if user set
    silent : bool
        if True print status comments or not
    debug : bool
        if True print RunTime warnings
        
        
    Returns
    -------
    -
    
    
    Output
    ------
    .txt and .pdf / .png files with all the centroid information
    
    
    Examples
    --------
    
    Example 1: if user wants to manually overwrite the BLS or CANVAS period, parse
    user_period (in s)
    user_epoch (in s)
    user_width (in s)
    
    Example 2: to run in the NGTS pipeline, parse
    output='pipeline'
    outfname_info='foo/bar/filename' (give full directory path)
    outfname_plot='foo/bar/plotname' (give full directory path)
    '''
    
    def __init__(self, fieldname, obj_id, ngts_version, source = 'BLS', \
                 bls_rank = 1, user_period = None, user_epoch = None, user_width=None, \
                 user_flux = None, user_centdx = None, user_centdy = None, time_hjd = None, \
                 pixel_radius = 200., flux_min = 500., flux_max = 10000., method='transit', \
                 R_min=0., N_top_max=20, bin_width=300., min_time=1800., dt=0.005, \
                 secondary_eclipse=True, blend_radius=6, \
                 roots=None, outdir=None, parent=None, \
                 do_plot='minimal', \
                 show_plot=False, flagfile=None, dic=None, nancut=None, \
                 output='minimal', \
                 fname_BLSPipe_megafile=None, outfname_info=None, outfname_plot=None, \
                 silent=True, debug=False):
        
        self.roots = roots
        
#        self.exposure = 12. #(in s)
        self.fieldname = fieldname
        self.obj_id = obj_id
        self.ngts_version = ngts_version
        self.source = source
        self.bls_rank = bls_rank
        self.user_period = user_period
        self.user_epoch = user_epoch
        self.user_width = user_width
        self.user_flux = user_flux
        self.user_centdx = user_centdx
        self.user_centdy = user_centdy
        self.time_hjd = time_hjd
        self.pixel_radius = pixel_radius
        self.flux_min = flux_min
        self.flux_max = flux_max
        self.method = method
        self.R_min = R_min
        self.N_top_max = N_top_max
        self.bin_width = bin_width #(in s)
        self.min_time = min_time #(in s) minimum coverage of innermost and out-of-transit required for including a night
        self.dt = dt
        self.secondary_eclipse = secondary_eclipse
        self.blend_radius = blend_radius
        self.do_plot = do_plot
        self.show_plot = show_plot
        self.flagfile = flagfile
        self.dic = dic
        self.nancut = nancut
        self.output = output
        self.fname_BLSPipe_megafile = fname_BLSPipe_megafile
        self.outfname_info = outfname_info
        self.outfname_plot = outfname_plot
        self.silent = silent
        self.debug = debug
        
        if self.debug is False:
            warnings.filterwarnings("ignore", category=RuntimeWarning)
        
        self.crosscorr = {}
        
        if outdir is None: 
#            self.outdir = os.path.join( 'output', self.ngts_version, self.fieldname, str(self.pixel_radius)+'_'+str(self.flux_min)+'_'+str(self.flux_max)+'_'+self.method+'_'+str(self.R_min)+'_'+str(self.N_top_max)+'_'+str(self.bin_width)+'_'+str(self.min_time)+'_'+str(self.dt), '' )
            self.outdir = os.path.join( 'output', self.ngts_version, self.fieldname, 'dt='+str(self.dt), '' )
        else:
            self.outdir = outdir
        if not os.path.exists(self.outdir): 
            os.makedirs(self.outdir)
          
        try:
            self.catalogfname = glob.glob( 'input/catalog/'+self.fieldname+'*'+self.ngts_version+'_cat_master.dat')[0]
        except:
            self.catalogfname = None
            
            
        #for in pipeline use:  
        if self.source=='pipeline':
            self.output = 'pipeline'
            self.do_plot = 'pipeline'
            self.fnames = {'BLSPipe_megafile': self.fname_BLSPipe_megafile}
        else: 
            self.fnames = None
            
#        self.run()
        
        
        
    ###########################################################################
    #::: load data (all nights) of the target
    ###########################################################################
    def load_object(self):
        
        #::: if no basic dic has been given, load it via ngtsio
        if self.dic is None:
            
            #::: set keys to be retrieved for the target object
            keys = ['OBJ_ID','FLUX_MEAN','RA','DEC', \
                        'NIGHT','AIRMASS', \
                        'HJD','CCDX','CCDY','CENTDX','CENTDY','SYSREM_FLUX3', \
                        'PERIOD','WIDTH','EPOCH','DEPTH','NUM_TRANSITS']
                    
            #if CANVAS is desired, attach CANVAS keys
            if self.source=='CANVAS':
                keys = keys + ['CANVAS_PERIOD','CANVAS_WIDTH','CANVAS_EPOCH','CANVAS_DEPTH']
                
            #::: use ngtsio to retrieve information
            self.dic = ngtsio.get(self.fieldname, self.ngts_version, keys, obj_id = self.obj_id, time_hjd = self.time_hjd, bls_rank = self.bls_rank, fnames=self.fnames, silent = True, set_nan = True)

            #if CANVAS is desired, overwrite BLS keys
            if self.source == 'CANVAS':
                #::: overwrite BLS with CANVAS infos (if existing) -> copy the keys
                for key in ['PERIOD','WIDTH','EPOCH','DEPTH']:
                    #::: if the CANVAS data exists, use it
                    if ('CANVAS_'+key in self.dic) and (~np.isnan( self.dic['CANVAS_'+key] )): 
                        self.dic[key] = self.dic['CANVAS_'+key].copy()
                        #::: convert CANVAS depth (in mmag) into the standard BLS depth (float)
                        if key == 'DEPTH': self.dic[key] *= -1E-3
                    #::: if the CANVAS data is missing, make BLS the source
                    else:
                        self.source = 'BLS'
        else:
            pass #(self.dic does already exist)
                    
        #::: set FLUX=0 entries to NaN
        # not needed anymore, now handled within ngtsio
#        self.dic = set_nan.set_nan(self.dic) 
        
        #::: calculate unique nights
        self.dic['UNIQUE_NIGHT'] = np.unique(self.dic['NIGHT']) 
        
        #::: calcualte median CCD position
        self.dic['CCDX_0'] = np.nanmedian(self.dic['CCDX'])
        self.dic['CCDY_0'] = np.nanmedian(self.dic['CCDY'])
        
        #::: overwrite flux, period, epoch and width if given by user
        #::: raise error if user_flux is not the correct length
        if self.user_flux is not None:
            if self.user_flux.shape != self.dic['SYSREM_FLUX3'].shape:
                raise ValueError( "user_flux must be the same length as the ngtsio_flux. self.user_flux.shape = "  + str(self.user_flux.shape) + ", self.dic['SYSREM_FLUX3'].shape = " + str(self.dic['SYSREM_FLUX3'].shape) )
            self.dic['SYSREM_FLUX3'] = self.user_flux
            
        if self.user_centdx is not None:
            if self.user_centdx.shape != self.dic['CENTDX'].shape:
                raise ValueError( "user_centdx must be the same length as the ngtsio_centdx. self.user_centdx.shape = "  + str(self.user_centdx.shape) + ", self.dic['CENTDX'].shape = " + str(self.dic['CENTDX'].shape) )
            self.dic['CENTDX'] = self.user_centdx
            
        if self.user_centdy is not None:
            if self.user_centdy.shape != self.dic['CENTDY'].shape:
                raise ValueError( "user_centdy must be the same length as the ngtsio_centdy. self.user_centdy.shape = "  + str(self.user_centdy.shape) + ", self.dic['CENTDY'].shape = " + str(self.dic['CENTDY'].shape) )
            self.dic['CENTDY'] = self.user_centdy
                
        if ( (self.user_flux is not None) | (self.user_centdx is not None) | (self.user_centdy is not None) ):
            self.dic = set_nan.set_nan(self.dic, key='SYSREM_FLUX3')
            
        if (self.user_period is not None) & (self.user_period > 0):
            self.dic['PERIOD'] = self.user_period #in s
        if (self.user_epoch is not None) & (self.user_epoch > 0):
            self.dic['EPOCH'] = self.user_epoch #in s
        if (self.user_width is not None) & (self.user_width > 0):
            self.dic['WIDTH'] = self.user_width #in s
            
                
        

    ###########################################################################
    #::: identify the neighbouring objects for reference
    ########################################################################### 
    def load_neighbours(self):
        
        #::: load position and flux of all objects in the field for reference
        self.dic_all = ngtsio.get(self.fieldname, self.ngts_version, ['CCDX','CCDY','FLUX_MEAN'], time_index=0, bls_rank = self.bls_rank, fnames=self.fnames, silent=True)
        
        
        
        #::: find neighbouring objects
        ind_neighbour = np.where( (np.abs(self.dic_all['CCDX'] - self.dic['CCDX_0']) < self.pixel_radius) & \
                                 (np.abs(self.dic_all['CCDY'] - self.dic['CCDY_0']) < self.pixel_radius) & \
                                 (self.dic_all['FLUX_MEAN'] > self.flux_min) & \
                                 (self.dic_all['FLUX_MEAN'] < self.flux_max) & \
                                 (self.dic_all['OBJ_ID'] != self.dic['OBJ_ID']) \
                                 )[0] 
        obj_id_nb = self.dic_all['OBJ_ID'][ind_neighbour]             
        
        #::: get infos of neighbouring objects
        self.dic_nb = ngtsio.get( self.fieldname, self.ngts_version, ['OBJ_ID','RA','DEC','HJD','CCDX','CCDY','CENTDX','CENTDY','FLUX_MEAN'], obj_id = obj_id_nb, time_hjd = self.time_hjd, bls_rank = self.bls_rank, fnames=self.fnames, silent = True) 
        self.dic_nb['CCDX_0'] = np.nanmedian( self.dic_nb['CCDX'], axis=1 )
        self.dic_nb['CCDY_0'] = np.nanmedian( self.dic_nb['CCDY'], axis=1 )
        
        
        
        #::: find blends (closeby and including faint and saturated stars) (only used for identify_blends.py)
        ind_blends = np.where( (np.abs(self.dic_all['CCDX'] - self.dic['CCDX_0']) < self.blend_radius) & \
                                (np.abs(self.dic_all['CCDY'] - self.dic['CCDY_0']) < self.blend_radius) & \
                                 (self.dic_all['OBJ_ID'] != self.dic['OBJ_ID']) \
                                 )[0]          
        obj_id_blends = list( self.dic_all['OBJ_ID'][ind_blends] )
        
        #::: get infos of blends
        if not obj_id_blends:
            self.dic_blends = None
        else:
            self.dic_blends = ngtsio.get( self.fieldname, self.ngts_version, ['OBJ_ID','RA','DEC','HJD','CCDX','CCDY','SYSREM_FLUX3','FLUX_MEAN'], obj_id = obj_id_blends, time_hjd = self.time_hjd, bls_rank = self.bls_rank, fnames=self.fnames, silent = True, simplify=False) 
            self.dic_blends['CCDX_0'] = np.nanmedian( self.dic_blends['CCDX'], axis=1 )
            self.dic_blends['CCDY_0'] = np.nanmedian( self.dic_blends['CCDY'], axis=1 ) 
            self.dic_blends['B-V'] = np.zeros( len(self.dic_blends['OBJ_ID']) ) * np.nan
            self.dic_blends['Vmag'] = np.zeros( len(self.dic_blends['OBJ_ID']) ) * np.nan
        
        #::: delete the dictionary of all objects
        del self.dic_all
        
        #::: apply same nancut as for target object to all neighbours 
        #::: ("autovet" specific only)
        if self.nancut is not None:
            for key in self.dic_nb: 
                if isinstance(self.dic_nb[key], np.ndarray):
                    if (self.dic_nb[key].ndim==2) :
                        self.dic_nb[key] = self.dic_nb[key][slice(None), ~self.nancut]
    


    ###########################################################################
    #::: import crossmatched catalog (Ed's version with 2MASS)
    ########################################################################### 
    def load_catalog(self):
        #::: if the catalogue for this object exists
        if self.catalogfname is not None:
            catdata = np.genfromtxt( self.catalogfname, names=True )
            
            self.dic['B-V'] = catdata['BV'][ catdata['NGTS_ID'] == float(self.dic['OBJ_ID']) ]   
            self.dic['Vmag'] = catdata['Vmag'][ catdata['NGTS_ID'] == float(self.dic['OBJ_ID']) ]  
            self.dic['Jmag'] = catdata['Jmag'][ catdata['NGTS_ID'] == float(self.dic['OBJ_ID']) ]  
            self.dic['Bmag'] = catdata['Bmag'][ catdata['NGTS_ID'] == float(self.dic['OBJ_ID']) ]  
            
            self.dic_nb['B-V'] = np.zeros( len(self.dic_nb['OBJ_ID']) ) * np.nan
            self.dic_nb['Vmag'] = np.zeros( len(self.dic_nb['OBJ_ID']) ) * np.nan
            for i, obj_id in enumerate(self.dic_nb['OBJ_ID']):
                self.dic_nb['B-V'][i] = catdata['BV'][ catdata['NGTS_ID'] == float(obj_id) ]  
                self.dic_nb['Vmag'][i] = catdata['Vmag'][ catdata['NGTS_ID'] == float(obj_id) ]  
               
            del catdata
        
        #::: otherwise
        else:
            self.dic['B-V'] = np.nan
            self.dic['Vmag'] = np.nan
            self.dic['Jmag'] = np.nan
            self.dic['Bmag'] = np.nan
            self.dic_nb['B-V'] = np.zeros( len(self.dic_nb['OBJ_ID']) ) * np.nan
            self.dic_nb['Vmag'] = np.zeros( len(self.dic_nb['OBJ_ID']) ) * np.nan



    def mark_eclipses(self):
        if self.secondary_eclipse is True:
            self.ind_out = index_transits.index_eclipses(self.dic)[-1]
        else:
            self.ind_out = index_transits.index_transits(self.dic)[3]
            


    def assign_airmass_colorcode(self):
        #::: assign colors for different nights; dconvert HJD from seconds into days
        self.dic = get_scatter_color.get_scatter_color(self.dic)
        self.dic['COLOR'] = np.concatenate( self.dic['COLOR_PER_NIGHT'], axis=0 )

        

    def binning(self):

        if self.do_plot=='all':
            #also bin COLOR and AIRMASS
            self.dic['HJD_BIN'], \
                [ self.dic['CENTDX_fda_BIN'], self.dic['CENTDY_fda_BIN'], self.dic['ma_CENTDX_BIN'], self.dic['ma_CENTDY_BIN'], self.dic['COLOR_BIN'],  self.dic['AIRMASS_BIN'], self.dic['SYSREM_FLUX3_BIN'], self.dic['CENTDX_f_BIN'], self.dic['CENTDY_f_BIN'], self.dic['CENTDX_fd_BIN'], self.dic['CENTDY_fd_BIN'], self.dic_nb['CENTDX_ref_mean_BIN'], self.dic_nb['CENTDY_ref_mean_BIN'] ], \
                [ self.dic['CENTDX_fda_BIN_ERR'], self.dic['CENTDY_fda_BIN_ERR'], self.dic['ma_CENTDX_BIN_ERR'], self.dic['ma_CENTDY_BIN_ERR'], self.dic['COLOR_BIN_ERR'], self.dic['AIRMASS_BIN_ERR'], self.dic['SYSREM_FLUX3_BIN_ERR'], self.dic['CENTDX_f_BIN_ERR'], self.dic['CENTDY_f_BIN_ERR'], self.dic['CENTDX_fd_BIN_ERR'], self.dic['CENTDY_fd_BIN_ERR'], self.dic_nb['CENTDX_ref_mean_BIN_ERR'], self.dic_nb['CENTDY_ref_mean_BIN_ERR'] ], \
                _ = lightcurve_tools.rebin_err_matrix(self.dic['HJD'], np.vstack(( self.dic['CENTDX_fda'], self.dic['CENTDY_fda'], self.dic['ma_CENTDX'], self.dic['ma_CENTDY'], self.dic['COLOR'],  self.dic['AIRMASS'], self.dic['SYSREM_FLUX3'], self.dic['CENTDX_f'], self.dic['CENTDY_f'], self.dic['CENTDX_fd'], self.dic['CENTDY_fd'], self.dic_nb['CENTDX_ref_mean'], self.dic_nb['CENTDY_ref_mean'] )), dt=600, sigmaclip=False, ferr_style='std' )

        else:            
            #no need for COLOR and AIRMASS
            self.dic['HJD_BIN'], \
                [ self.dic['CENTDX_fda_BIN'], self.dic['CENTDY_fda_BIN'], self.dic['ma_CENTDX_BIN'], self.dic['ma_CENTDY_BIN'], self.dic['SYSREM_FLUX3_BIN'], self.dic['CENTDX_f_BIN'], self.dic['CENTDY_f_BIN'], self.dic['CENTDX_fd_BIN'], self.dic['CENTDY_fd_BIN'], self.dic_nb['CENTDX_ref_mean_BIN'], self.dic_nb['CENTDY_ref_mean_BIN'] ], \
                [ self.dic['CENTDX_fda_BIN_ERR'], self.dic['CENTDY_fda_BIN_ERR'], self.dic['ma_CENTDX_BIN_ERR'], self.dic['ma_CENTDY_BIN_ERR'], self.dic['SYSREM_FLUX3_BIN_ERR'], self.dic['CENTDX_f_BIN_ERR'], self.dic['CENTDY_f_BIN_ERR'], self.dic['CENTDX_fd_BIN_ERR'], self.dic['CENTDY_fd_BIN_ERR'], self.dic_nb['CENTDX_ref_mean_BIN_ERR'], self.dic_nb['CENTDY_ref_mean_BIN_ERR'] ], \
                _ = lightcurve_tools.rebin_err_matrix(self.dic['HJD'], np.vstack(( self.dic['CENTDX_fda'], self.dic['CENTDY_fda'], self.dic['ma_CENTDX'], self.dic['ma_CENTDY'], self.dic['SYSREM_FLUX3'], self.dic['CENTDX_f'], self.dic['CENTDY_f'], self.dic['CENTDX_fd'], self.dic['CENTDY_fd'], self.dic_nb['CENTDX_ref_mean'], self.dic_nb['CENTDY_ref_mean'] )), dt=600, sigmaclip=False, ferr_style='std' )



    ###########################################################################
    #::: 
    ########################################################################### 
    def phase_fold(self):
        self.N_phasepoints = int( 1./self.dt )
        
        self.dic['HJD_PHASE'], self.dic['SYSREM_FLUX3_PHASE'], self.dic['SYSREM_FLUX3_PHASE_ERR'], self.dic['N_PHASE'], self.dic['PHI'] = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['SYSREM_FLUX3'] / np.nanmedian(self.dic['SYSREM_FLUX3'][self.ind_out]), self.dic['PERIOD'], self.dic['EPOCH'], dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
        _, self.dic['CENTDX_PHASE'], self.dic['CENTDX_PHASE_ERR'], _, _ = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['CENTDX'] - np.nanmedian(self.dic['CENTDX'][self.ind_out]), self.dic['PERIOD'], self.dic['EPOCH'], dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
        _, self.dic['CENTDY_PHASE'], self.dic['CENTDY_PHASE_ERR'], _, _ = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['CENTDY'] - np.nanmedian(self.dic['CENTDY'][self.ind_out]), self.dic['PERIOD'], self.dic['EPOCH'], dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
        _, self.dic['CENTDX_f_PHASE'], self.dic['CENTDX_f_PHASE_ERR'], _, _ = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['CENTDX_f'] - np.nanmedian(self.dic['CENTDX_f'][self.ind_out]), self.dic['PERIOD'], self.dic['EPOCH'], dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
        _, self.dic['CENTDY_f_PHASE'], self.dic['CENTDY_f_PHASE_ERR'], _, _ = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['CENTDY_f'] - np.nanmedian(self.dic['CENTDY_f'][self.ind_out]), self.dic['PERIOD'], self.dic['EPOCH'], dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
        _, self.dic['CENTDX_fd_PHASE'], self.dic['CENTDX_fd_PHASE_ERR'], _, _ = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['CENTDX_fd'] - np.nanmedian(self.dic['CENTDX_fd'][self.ind_out]), self.dic['PERIOD'], self.dic['EPOCH'], dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
        _, self.dic['CENTDY_fd_PHASE'], self.dic['CENTDY_fd_PHASE_ERR'], _, _ = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['CENTDY_fd'] - np.nanmedian(self.dic['CENTDY_fd'][self.ind_out]), self.dic['PERIOD'], self.dic['EPOCH'], dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
        _, self.dic['CENTDX_fda_PHASE'], self.dic['CENTDX_fda_PHASE_ERR'], _, _ = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['CENTDX_fda'] - np.nanmedian(self.dic['CENTDX_fda'][self.ind_out]), self.dic['PERIOD'], self.dic['EPOCH'], dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
        _, self.dic['CENTDY_fda_PHASE'], self.dic['CENTDY_fda_PHASE_ERR'], _, _ = lightcurve_tools.phase_fold(self.dic['HJD'], self.dic['CENTDY_fda'] - np.nanmedian(self.dic['CENTDY_fda'][self.ind_out]), self.dic['PERIOD'], self.dic['EPOCH'], dt = self.dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)

        
        
    ###########################################################################
    #::: 
    ########################################################################### 
    def cross_correlate(self):
        #::: create pandas df from parts of self.dic
        self.phasedf = pd.DataFrame( {k: self.dic[k] for k in ('HJD_PHASE', 'SYSREM_FLUX3_PHASE', 'CENTDX_fda_PHASE', 'CENTDY_fda_PHASE')}, columns=['HJD_PHASE', 'CENTDX_fda_PHASE', 'CENTDY_fda_PHASE', 'SYSREM_FLUX3_PHASE'] )
   
        self.fig_corrfx, flags_fx = self.ccfct( 'SYSREM_FLUX3_PHASE', 'CENTDX_fda_PHASE', 'FLUX vs CENTDX' )
        self.fig_corrfy, flags_fy = self.ccfct( 'SYSREM_FLUX3_PHASE', 'CENTDY_fda_PHASE', 'FLUX vs CENTDY' )
        self.fig_corrxy, flags_xy = self.ccfct( 'CENTDX_fda_PHASE', 'CENTDY_fda_PHASE', 'CENTDX vs CENTDY' )

	if self.do_plot=='all':
            self.fig_autocorr = self.acfct( ['SYSREM_FLUX3_PHASE','CENTDX_fda_PHASE','CENTDY_fda_PHASE'], ['FLUX','CENTDX','CENTDY'] )            
             
 
    #::: autocorrelation fct
    def acfct(self, xkeys, titles):
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        fig, axes = plt.subplots(2,3,figsize=(15,8))
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
               
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#        for i, xkey in enumerate(xkeys):
#            x = self.phasedf[xkey]
#            ax = axes[0,i]
#            
#            autocorr, lags, autocorr_CI95, autocorr_CI99 = pandas_tsa.pandas_autocorr(x)
#            ax.plot( lags, autocorr )
#            ax.plot( lags[10:-10], autocorr_CI99[10:-10], 'k--' )
#            ax.plot( lags[10:-10], -autocorr_CI99[10:-10], 'k--' )
#            ax.set( xlim=[lags[0]-50, lags[-1]+50], ylim=[-1,1], xlabel=r'lag $\tau$ (phase shift)', ylabel='acf' )
#            ax.set_xticklabels( [(j*self.dt) for j in ax.get_xticks()] )
#            ax.set_title( titles[i] )
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::

        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#        for i, xkey in enumerate(xkeys):
#            x = self.phasedf[xkey]
#            ax = axes[1,i]
#            
#            kwargs = { 'marker':None, 'linestyle':'-'}
#            sm.graphics.tsa.plot_acf(x, ax=ax, alpha=0.05, lags=len(x)-1, unbiased=True, use_vlines=False, **kwargs)
#            lags = np.arange( len(x) )
#            autocorr_CI95 = 1.96/np.sqrt( len(x) - lags )
#            autocorr_CI99 = 2.58/np.sqrt( len(x) - lags )
#            ax.plot( lags[0], autocorr[0], 'o' )
#            ax.plot( lags[10:-10], autocorr_CI99[10:-10], 'k--' )
#            ax.plot( lags[10:-10], -autocorr_CI99[10:-10], 'k--' )
#            ax.set( xlim=[lags[0]-50, lags[-1]+50], ylim=[-1,1], xlabel=r'lag $\tau$ (phase shift)', ylabel='acf', title='' )
#            ax.set_xticklabels( [(j*self.dt) for j in ax.get_xticks()] )
#            ax.legend()
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::

        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        for i, xkey in enumerate(xkeys):
            ax = axes[0,i]
            x = self.phasedf[xkey]
            autocorr, lags, autocorr_CI95, autocorr_CI99 = pandas_tsa.pandas_periodic_autocorr(x)
            N_a = len(autocorr)   
            xlags = np.linspace(-0.25,0.75,N_a)   
            autocorr = np.concatenate( (autocorr[ int(3*N_a/4): ], autocorr[ :int(3*N_a/4) ]) )
            autocorr_CI95 = np.concatenate( (autocorr_CI95[ int(3*N_a/4): ], autocorr_CI95[ :int(3*N_a/4) ]) )
            autocorr_CI99 = np.concatenate( (autocorr_CI99[ int(3*N_a/4): ], autocorr_CI99[ :int(3*N_a/4) ]) )
            ax.plot( xlags, autocorr, 'g-' )
            ax.plot( xlags[10:-10], autocorr_CI99[10:-10], 'k--' )
            ax.plot( xlags[10:-10], -autocorr_CI99[10:-10], 'k--' )
            ax.set( title=titles[i], xlim=[xlags[0], xlags[-1]], ylim=[-1,1], xlabel=r'lag $\tau$ (phase shift)', ylabel='acf (periodic)' )
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::        
            
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        for i, xkey in enumerate(xkeys):
            x = self.phasedf[xkey]
            ax = axes[1,i]
            
#            kwargs = { 'marker':None, 'linestyle':'-'}
            nlags = 50
#            try:
#                sm.graphics.tsa.plot_pacf(x, ax=ax, alpha=0.05, lags=nlags, use_vlines=False, **kwargs) #lags=len(x)-1
#            except:
#                pass
            lags = np.arange( nlags )
            ax.set( xlim=[lags[0]-2, lags[-1]+2], ylim=[-1,1], xlabel=r'lag $\tau$ (phase shift)', ylabel='pacf', title='' )
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
                   
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        plt.tight_layout()
        return fig
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
       
        
               
             
    def ccfct(self, xkey, ykey, title):
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        x = self.phasedf[xkey]
        y = self.phasedf[ykey]
        
        if self.do_plot in ('minimal','all'):
            fig, axes = plt.subplots(1,2,figsize=(10,4))
            fig.suptitle(self.obj_id + ' ' + title)
        else:
            fig = None
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
               
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#        win = [0.1, 0.25, 0.33]
        win = [0.25]
        windows= ( np.array(win) * (1/self.dt) ).astype(int)
#        correls = [ self.phasedf.rolling(window=windows[i], center=True, min_periods=1).corr() for i,_ in enumerate(windows) ]
        
#        color = ['b','g','r']
        for i,_ in enumerate(windows): 
#            self.dic['RollCorr_'+xkey+'_'+ykey] = correls[i].loc[ :, xkey, ykey ] #leads to indexing errors in pandas version > 0.18 for mysterious reasons...
            self.dic['RollCorr_'+xkey+'_'+ykey] = self.phasedf[xkey].rolling(window=windows[i], center=True, min_periods=1).corr(self.phasedf[ykey])
            self.rollcorr_CI99 = 2.58/np.sqrt(windows[i])
            if self.do_plot in ('minimal','all'):
                axes[0].plot( self.phasedf['HJD_PHASE'], self.dic['RollCorr_'+xkey+'_'+ykey], label=str(windows[i] * self.dt) )
                axes[0].axhline( self.rollcorr_CI99, color='k', linestyle='--')#, color=color[i] )
                axes[0].axhline( -self.rollcorr_CI99, color='k', linestyle='--')#, color=color[i] )
                axes[0].set( xlim=[-0.25,0.75], ylim=[-1,1], xlabel='phase', ylabel='rolling correlation')
                axes[0].legend()
                
#        crosscorr, lags, crosscorr_CI95, crosscorr_CI99 = pandas_tsa.pandas_crosscorr(x, y)
#        axes[1].plot( lags, crosscorr )
#        axes[1].plot( lags[10:-10], crosscorr_CI99[10:-10], 'k--' )
#        axes[1].plot( lags[10:-10], -crosscorr_CI99[10:-10], 'k--' )
#        axes[1].set( xlim=[lags[0]-50, lags[-1]+50], ylim=[-1,1], xlabel=r'lag $\tau$ (phase shift)', ylabel='ccf' )
#        axes[1].set_xticklabels( [(i*self.dt) for i in axes[1].get_xticks()] )
            
        crosscorr, lags, crosscorr_CI95, crosscorr_CI99 = pandas_tsa.pandas_periodic_crosscorr(x, y)
        N_c = len(crosscorr)   
        self.ccf_lags = np.linspace(-0.25,0.75,N_c)
        self.crosscorr[title] = np.concatenate( (crosscorr[ int(3*N_c/4): ], crosscorr[ :int(3*N_c/4) ]) )
        self.dic['CrossCorr_'+xkey+'_'+ykey] = self.crosscorr[title]
        self.crosscorr_CI95 = np.concatenate( (crosscorr_CI95[ int(3*N_c/4): ], crosscorr_CI95[ :int(3*N_c/4) ]) )
        self.crosscorr_CI99 = np.concatenate( (crosscorr_CI99[ int(3*N_c/4): ], crosscorr_CI99[ :int(3*N_c/4) ]) )
        if self.do_plot in ('minimal','all'):
            axes[1].plot( self.ccf_lags, self.crosscorr[title] )
            axes[1].plot( self.ccf_lags, self.crosscorr_CI99, 'k--' )
            axes[1].plot( self.ccf_lags, - self.crosscorr_CI99, 'k--' )
            axes[1].set( xlim=[self.ccf_lags[0], self.ccf_lags[-1]], ylim=[-1,1], xlabel=r'lag $\tau$ (phase shift)', ylabel='ccf (periodic)' )
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
                  
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        if self.do_plot in ('minimal','all'):
            plt.tight_layout() 
        return fig, None
        #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::

            

       
    def do_stats(self):
        
        #::: center on out-of-transit
        phi = self.dic['HJD_PHASE']
        ind_0 = np.argmin(np.abs(phi)) #i.e. where PHASE==0
        #TODO: very rough approximation
        ind_out_phase = np.where( (phi<-0.15) | ((phi>0.15) & (phi<0.35)) | (phi>0.65) )[0] #::: very rough approx.
#            ind_in_transit = np.where( self.dic['SYSREM_FLUX3_PHASE']<0.99 )[0] #::: very rough approx.
        centdx = self.dic['CENTDX_fda_PHASE']*1000.
        centdy = self.dic['CENTDY_fda_PHASE']*1000.
        centdx -= np.nanmedian(centdx[ ind_out_phase ])
        centdy -= np.nanmedian(centdy[ ind_out_phase ])

        #::: open dictionary on correlation stats
        self.stats = {}   
        
        #::: Signal-to-noise ratio in rolling correlation (noise == standard deviation out of transit) 
        for key in ['X','Y']:
            self.stats['RollCorrSNR_'+key] = np.abs( self.dic['RollCorr_SYSREM_FLUX3_PHASE_CENTD'+key+'_fda_PHASE'][ ind_0 ] / np.nanstd( self.dic['RollCorr_SYSREM_FLUX3_PHASE_CENTD'+key+'_fda_PHASE'][ ind_out_phase ] ) )
#            print SNR_RC
        
        #::: Signal-to-noise ratio in cross-correlation  
        for key in ['X','Y']:
            self.stats['CrossCorrSNR_'+key] = np.abs( self.dic['CrossCorr_SYSREM_FLUX3_PHASE_CENTD'+key+'_fda_PHASE'][ ind_0 ] / np.nanstd( self.dic['CrossCorr_SYSREM_FLUX3_PHASE_CENTD'+key+'_fda_PHASE'][ ind_out_phase ] ) )
#            print SNR_CC            
        
        #::: Hypothesis tests on rain plots
        for centd, key in zip([centdx, centdy], ['X','Y']):
            x_hyptest = len( centd[ (self.dic['SYSREM_FLUX3_PHASE']<0.99) & (centd<0.) ] )
            N_hyptest = len( centd[ self.dic['SYSREM_FLUX3_PHASE']<0.99 ] )
            self.stats['Binom_'+key] = binom_test( x_hyptest, N_hyptest, p=0.5 ) 
#                print 'p-value(Binom_' + key + ') = ' + str(p_value['Binom_'+key])   
#                if p_value['Binom_'+key] <= 0.01:
#                    print '\tREJECT the Null Hypothesis that there is no centroid shift.'
#                else:
#                    print '\tNo conclusion.'
                
            t_statistic, self.stats['Ttest_'+key] = ttest_1samp( centd[ self.dic['SYSREM_FLUX3_PHASE']<0.99 ], 0 )   
#                print 'p-value(Ttest_' + key + ') = ' + str(p_value['Ttest_'+key])   
#                if p_value['Ttest_'+key] <= 0.01:
#                    print '\tREJECT the Null Hypothesis that there is no centroid shift.'
#                else:
#                    print '\tNo conclusion.'
                         
              
              
    def make_suggestion(self):
 
        #TODO: very rough, btu BLS WIDTH is not reliable either (potentially even worse)
        ind_primary = np.where( (self.dic['HJD_PHASE'] > -0.05) & (self.dic['HJD_PHASE'] < 0.05) )[0]
        
        self.centd_flag = {'X':0, 'Y':0}
        '''
        if flag==0: no centd shift detected
        if flag==1: yellow warning
        if flag==2: red warning
        '''
        for key in ['X','Y']:
            for sigma, pvalue in [[5,0.05],[7,0.01]]:
                self.centd_flag[key] += int((self.stats['RollCorrSNR_'+key] > sigma)
                                        or (self.stats['CrossCorrSNR_'+key] > sigma)
                                        or (self.stats['Ttest_'+key] < pvalue)
                                        or (self.stats['Binom_'+key] < pvalue))
                     
        '''
        Note: opis image is flipped in y compared to the CCD image and CENTDXY!
        '''
#        for text, sigma, p_min in [['(potential) ',5,0.05],['',7,0.01]]:
                
        if self.centd_flag['Y']>0: 
            if np.nanmedian(self.dic['CENTDY_fda_PHASE'][ind_primary]) > 0:       
                updown = ' upper '
            else:
                updown = ' lower '
        else:
            updown = ''
                         
        if self.centd_flag['X']>0:
            if np.nanmedian(self.dic['CENTDX_fda_PHASE'][ind_primary]) > 0:       
                leftright = ' left '    
            else:
                leftright = ' right '
        else:
            leftright = ''
            
        if (self.centd_flag['X']==0) & (self.centd_flag['Y']==0):  
            self.suggestion = 'No centroid shift flagged automatically.' 
        elif (self.centd_flag['X']==2) | (self.centd_flag['Y']==2): 
            self.suggestion = 'Centd shift:' + updown + leftright + 'object eclipses (in opis sky image).'
        else:
            self.suggestion = 'Potential centd shift:' + updown + leftright + 'object eclipses (in opis sky image).'
            
            
            
                
    def plot_scatter_matrix(self):  
        
        try:
            axes = pd.plotting.scatter_matrix(self.phasedf[ ['CENTDX_fda_PHASE', 'CENTDY_fda_PHASE', 'SYSREM_FLUX3_PHASE'] ], alpha=0.6, figsize=(10, 10), diagonal='kde') # diagonal='hist'
        except:
            axes = pd.tools.plotting.scatter_matrix(self.phasedf[ ['CENTDX_fda_PHASE', 'CENTDY_fda_PHASE', 'SYSREM_FLUX3_PHASE'] ], alpha=0.6, figsize=(10, 10), diagonal='kde') # diagonal='hist'

        self.fig_matrix = plt.gcf()
        
        #Change label notation
        labels = ['CENTDX','CENTDY','FLUX']
        [s.xaxis.label.set_text(labels[i%3]) for i,s in enumerate(axes.reshape(-1))]
        [s.yaxis.label.set_text(labels[i/3]) for i,s in enumerate(axes.reshape(-1))]
    
        #::: remove dublicates form plot
        corr = self.phasedf[ ['CENTDX_fda_PHASE', 'CENTDY_fda_PHASE', 'SYSREM_FLUX3_PHASE'] ].corr().as_matrix()
        for n in range( axes.shape[0] * axes.shape[1] ):
            i, j = np.unravel_index(n, axes.shape)
            if j==0:
                axes[i,j].axvline(0, linestyle='--', color='grey')
                axes[i,j].axvline(0.5, linestyle='--', color='grey')
            if j>0:
                axes[i,j].axvline(0, linestyle='--', color='grey')
            # remove dublicates form plot
            if j>i: 
                axes[i,j].cla()
                axes[i,j].set_axis_off()
            # log-scale the kde plots
            if i == j:  
                axes[i,j].set_yscale('log')
            # annotate corr coefficients
            if j<i:
                axes[i,j].annotate("%.3f" %corr[i,j], (0.85, 0.9), xycoords='axes fraction', ha='center', va='center')
            


    ###########################################################################
    #::: look at phase-folded lightcurve and centroid curve
    ###########################################################################        
    def plot_phase_folded_curves(self):
        
        #::: detrended curves
        self.fig_phasefold, axes = plt.subplots( 4, 1, sharex=True, figsize=(16,16) )

        axes[0].plot( self.dic['PHI'], self.dic['SYSREM_FLUX3']/np.nanmedian(self.dic['SYSREM_FLUX3']), 'k.', alpha=0.1, rasterized=True )
        axes[0].errorbar( self.dic['HJD_PHASE'], self.dic['SYSREM_FLUX3_PHASE'], yerr=self.dic['SYSREM_FLUX3_PHASE_ERR'], fmt='o', color='r', ms=10, rasterized=True )
        axes[0].set( ylabel='FLUX', ylim=[ np.nanmin(self.dic['SYSREM_FLUX3_PHASE']-self.dic['SYSREM_FLUX3_PHASE_ERR']), np.nanmax(self.dic['SYSREM_FLUX3_PHASE']+self.dic['SYSREM_FLUX3_PHASE_ERR']) ])
        
        axes[1].plot( self.dic['PHI'], self.dic['CENTDX'], 'k.', alpha=0.1, rasterized=True )
        axes[1].errorbar( self.dic['HJD_PHASE'], self.dic['CENTDX_fda_PHASE'], yerr=self.dic['CENTDX_fda_PHASE_ERR'], fmt='o', ms=10, rasterized=True ) #, color='darkgrey')
        axes[1].set( ylabel='CENTDX (in pixel)', ylim=[ np.nanmin(self.dic['CENTDX_fda_PHASE']-self.dic['CENTDX_fda_PHASE_ERR']), np.nanmax(self.dic['CENTDX_fda_PHASE']+self.dic['CENTDX_fda_PHASE_ERR']) ])
        
        axes[2].plot( self.dic['PHI'], self.dic['CENTDY'], 'k.', alpha=0.1, rasterized=True )
        axes[2].errorbar( self.dic['HJD_PHASE'], self.dic['CENTDY_fda_PHASE'], yerr=self.dic['CENTDY_fda_PHASE_ERR'], fmt='o', ms=10, rasterized=True ) #, color='darkgrey')
        axes[2].set( ylabel='CENTDY (in pixel)', xlabel='Phase', ylim=[ np.nanmin(self.dic['CENTDY_fda_PHASE']-self.dic['CENTDY_fda_PHASE_ERR']), np.nanmax(self.dic['CENTDY_fda_PHASE']+self.dic['CENTDY_fda_PHASE_ERR']) ], xlim=[-0.25,0.75])
        
        axes[3].plot( self.dic['HJD_PHASE'], self.dic['N_PHASE'], 'go', ms=10, rasterized=True ) #, color='darkgrey')
        axes[3].set( ylabel='Nr of exposures', xlabel='Phase', xlim=[-0.25,0.75])
        
        plt.tight_layout()
        
           
         
    ###########################################################################
    #::: plot info page
    ###########################################################################  
    def plot_info_page(self):
        #::: plot object
        self.fig_info_page = plt.figure(figsize=(16,3.6))
        gs = gridspec.GridSpec(1, 4)
        
        #::: plot locations on CCD    
        ax = plt.subplot(gs[0, 0])
        xtext = 50
        if self.dic['CCDY_0'] < 1000:
            ytext = 1800
        else:
            ytext = 350
        ax.text(xtext,ytext,'Flux ' + mystr(self.flux_min) + '-' + mystr(self.flux_max))
        ax.text(xtext,ytext-250, str(self.pixel_radius) + ' px')
        ax.plot( self.dic_nb['CCDX_0'], self.dic_nb['CCDY_0'], 'k.', rasterized=True )
        ax.plot( self.dic['CCDX_0'], self.dic['CCDY_0'], 'r.', rasterized=True ) 
        ax.add_patch( patches.Rectangle(
                        (self.dic['CCDX_0']-self.pixel_radius, self.dic['CCDY_0']-self.pixel_radius),
                        2*self.pixel_radius,
                        2*self.pixel_radius,
                        fill=False, color='r', lw=2) )
    #    ax.axis('equal')
        ax.set(xlim=[0,2048], ylim=[0,2048], xlabel='CCDX', ylabel='CCDY') 
        
        #::: plot lightcurve
        ax = plt.subplot(gs[0, 1:3])
        time = self.dic['HJD'] / 3600. / 24.
        ax.plot( time, self.dic['SYSREM_FLUX3'], 'k.', rasterized=True )
        ax.set(xlim=[time[0], time[-1]], xlabel='HJD', ylabel='Flux (counts)')
        
        #::: add second axis to lightcurve
        conversion = 1./np.nanmedian(self.dic['SYSREM_FLUX3'])
        ax2 = ax.twinx()
        mn, mx = ax.get_ylim()
        ax2.set_ylim(mn*conversion, mx*conversion)
        ax2.set_ylabel('Flux (norm.)')

        #::: plot info text
        ax = plt.subplot(gs[0, 3])
        plotting.plot_target_info_text(ax, self)    
        
        plt.tight_layout()
    
    
    
#    def plot_info_text(self, ax):
#        if 'DEPTH' not in self.dic: depth = ''
#        else: depth = mystr(np.abs(self.dic['DEPTH'])*1000.,2)
#        if 'NUM_TRANSITS' not in self.dic: num_transits = ''
#        else: num_transits = mystr(self.dic['NUM_TRANSITS'],0)
#            
#        ax.set_xlim([0,1])
#        ax.set_ylim([0,1])
#        ax.axis('off')
#        hmsdms = deg2hmsdms(self.dic['RA']*180/np.pi, self.dic['DEC']*180/np.pi)
##        ra, dec = deg2HMS.deg2HMS(ra=self.dic['RA'], dec=self.dic['DEC'])
#        ax.text(0,1.0,self.fieldname+' '+self.dic['OBJ_ID']+' '+self.source)
#        ax.text(0,0.9,hmsdms)
#        ax.text(0,0.8,'FLUX: '+str(self.dic['FLUX_MEAN']))
#        ax.text(0,0.7,'J-mag: '+mystr(self.dic['Jmag'],3))
#        ax.text(0,0.6,'V-mag: '+mystr(self.dic['Vmag'],3))
#        ax.text(0,0.5,'B-mag: '+mystr(self.dic['Bmag'],3))
#        ax.text(0,0.4,'B-V color: '+mystr(self.dic['B-V'],3))
##        ax.text(0,0.6,'PERIOD (s): '+mystr(self.dic['PERIOD'],2))
#        ax.text(0,0.3,'PERIOD (d): '+mystr(self.dic['PERIOD']/3600./24.,3))
##        ax.text(0,0.4,'Width (s): '+mystr(self.dic['WIDTH'],2))
#        ax.text(0,0.2,'Width (h): '+mystr(self.dic['WIDTH']/3600.,2))
##        ax.text(0,0.2,'EPOCH (s): '+mystr(self.dic['EPOCH'],2))
#        ax.text(0,0.1,'Depth (mmag): '+depth)
#        ax.text(0,0.0,'Num Transits: '+num_transits)
        


    ###########################################################################
    #::: look at phase-folded lightcurve and centroid curve
    ########################################################################### 
    def plot_stacked_image(self):
        #TODO: astropy version errors
#        try:
#            self.fig_stacked_image = stacked_images.plot(self.fieldname, self.ngts_version, self.dic['CCDX_0'], self.dic['CCDY_0'], r=15) 
#        except:
#            self.fig_stacked_image = None
        self.fig_stacked_image = None
            


    ###########################################################################
    #::: phase-folded lightcurve and centroid curve + all statistics
    ###########################################################################        
    def plot_for_pipeline(self):
        
        self.fig_pipeline, axes = plt.subplots( 5, 2, sharex=True, figsize=(11.5,11.5) )
        plt.suptitle(self.suggestion)
        
        for col, ccdax in [[0,'X'], [1,'Y']]:
            xkey = 'SYSREM_FLUX3_PHASE'
            centd = 'CENTD'+ccdax
            ykey = centd+'_fda_PHASE'
            
            axes[0,col].set(title='X')
            
            axes[0,col].errorbar( self.dic['HJD_PHASE'], self.dic['SYSREM_FLUX3_PHASE'], yerr=self.dic['SYSREM_FLUX3_PHASE_ERR'], fmt='o', color='k', ms=6, rasterized=True )
            axes[0,col].set( ylabel='FLUX', ylim=[ np.nanmin(self.dic['SYSREM_FLUX3_PHASE']-self.dic['SYSREM_FLUX3_PHASE_ERR']), np.nanmax(self.dic['SYSREM_FLUX3_PHASE']+self.dic['SYSREM_FLUX3_PHASE_ERR']) ])
            
            axes[1,col].errorbar( self.dic['HJD_PHASE'], 1e3*self.dic[ykey], yerr=1e3*self.dic[ykey+'_ERR'], fmt='o', color='k', ms=6, rasterized=True )
            axes[1,col].set( ylabel='CENTD (mpix)', ylim=[ 1e3*np.nanmin(self.dic[ykey]-self.dic[ykey+'_ERR']), 1e3*np.nanmax(self.dic[ykey]+self.dic[ykey+'_ERR']) ])
            
            axes[2,col].plot( self.dic['HJD_PHASE'], self.dic['RollCorr_'+xkey+'_'+ykey], 'k-' )
            axes[2,col].axhline( self.rollcorr_CI99, color='k', linestyle='--')
            axes[2,col].axhline( -self.rollcorr_CI99, color='k', linestyle='--')
            axes[2,col].set( xlim=[-0.25,0.75], ylim=[-1,1], ylabel='Roll. Corr.')

            axes[3,col].plot( self.ccf_lags, self.dic['CrossCorr_'+xkey+'_'+ykey], 'k-' )
            axes[3,col].plot( self.ccf_lags, self.crosscorr_CI99, 'k--' )
            axes[3,col].plot( self.ccf_lags, - self.crosscorr_CI99, 'k--' )
            axes[3,col].set( xlim=[-0.25,0.75], ylim=[-1,1], ylabel='Cross. Corr.')
            
            for i in range(4):
                axes[i,0].locator_params(nbins=3, axis='y')
                axes[i,1].locator_params(nbins=3, axis='y')

            self.plot_stats(axes[4,col], ccdax)
            
        for row in range(4):
            axes[row,1].set(ylabel='')
            
        plt.tight_layout()
        plt.subplots_adjust(top=0.9)
        
        
        
    
    def plot_stats(self, ax, ccdax):
        
        ax.set_xlim([-0.25,0.75])
        ax.set_ylim([0,1])
        ax.axis('off')
        
        def color_SNR(a):
            if a >= 7: return 'r'
            if a >= 5: return 'orange'
            if a < 5: return 'k'
            else: return 'k'
            
        def color_p(a):
            if a >= 0.05: return 'k'
            if a >= 0.01: return 'orange'
            if a < 0.01: return 'r'
            else: return 'k'
            
        for i,key in enumerate(['RollCorrSNR', 'CrossCorrSNR']):        
            ax.text(-0.25,1.-i/4.,key+': '+mystr(self.stats[key+'_'+ccdax],2), color=color_SNR(self.stats[key+'_'+ccdax]))
        for i,key in enumerate(['Ttest', 'Binom']):        
            ax.text(-0.25,.5-i/4.,key+': '+mystr(self.stats[key+'_'+ccdax],4), color=color_p(self.stats[key+'_'+ccdax]))
                    

      

    ###########################################################################
    #::: save all plots in one pdf per target object
    ###########################################################################   
    def save_pdf(self):
        outfilename = os.path.join( self.outdir, self.fieldname + '_' + self.obj_id + '_' + self.ngts_version + '_centroid_analysis.pdf' )          
        with PdfPages( outfilename ) as pdf:
            pdf.savefig( self.fig_phasefold  )
            pdf.savefig( self.fig_corrfx  )
            pdf.savefig( self.fig_corrfy  )
            pdf.savefig( self.fig_corrxy  )
            pdf.savefig( self.fig_matrix  )
            if self.do_plot=='all':
                pdf.savefig( self.fig_autocorr  )
            pdf.savefig( self.fig_info_page  )
            if self.fig_stacked_image is not None: 
                pdf.savefig( self.fig_stacked_image  )
            if not self.silent:
                print 'Plots saved as ' + outfilename
            
        if self.show_plot == False: 
            plt.close('all')
        else:
            plt.show()
            


    ###########################################################################
    #::: save all plots in individual subplot pngs (for pipeline use)
    ###########################################################################   
    def save_png(self):
        if self.outfname_plot is None:
            outfilename = os.path.join( self.outdir, self.fieldname + '_' + self.obj_id + '_' + self.ngts_version + '_centroid_analysis.pdf' )          
        else:
            outfilename = self.outfname_plot
        if not outfilename.endswith('.png'): 
            outfilename += '.png'
            
        self.fig_pipeline.savefig(outfilename, dpi=100)
        
        if self.show_plot == False: 
            plt.close('all')
        else:
            plt.show()
                
                
            
    ###########################################################################
    #::: save the phasecurve data for further external fitting
    ########################################################################### 
    def save_data(self):
        
        #::: phase-folded data
        outfilename = os.path.join( self.outdir, self.fieldname + '_' + self.obj_id + '_' + self.ngts_version + '_centroid_data_PHASE.txt' )
        X = np.c_[ self.dic['HJD_PHASE'], self.dic['SYSREM_FLUX3_PHASE'], self.dic['SYSREM_FLUX3_PHASE_ERR'], 
                   self.dic['CENTDX_fda_PHASE'], self.dic['CENTDX_fda_PHASE_ERR'], self.dic['CENTDY_fda_PHASE'], self.dic['CENTDY_fda_PHASE_ERR'], 
                   self.dic['CENTDX_fd_PHASE'], self.dic['CENTDX_fd_PHASE_ERR'], self.dic['CENTDY_fd_PHASE'], self.dic['CENTDY_fd_PHASE_ERR'], 
                   self.dic['CENTDX_f_PHASE'], self.dic['CENTDX_f_PHASE_ERR'], self.dic['CENTDY_f_PHASE'], self.dic['CENTDY_f_PHASE_ERR'], 
                   self.dic['CENTDX_PHASE'], self.dic['CENTDX_PHASE_ERR'], self.dic['CENTDY_PHASE'], self.dic['CENTDY_PHASE_ERR'],
                   self.dic['RollCorr_SYSREM_FLUX3_PHASE_CENTDX_fda_PHASE'], self.dic['RollCorr_SYSREM_FLUX3_PHASE_CENTDY_fda_PHASE'], self.dic['RollCorr_CENTDX_fda_PHASE_CENTDY_fda_PHASE'],
                   self.dic['CrossCorr_SYSREM_FLUX3_PHASE_CENTDX_fda_PHASE'], self.dic['CrossCorr_SYSREM_FLUX3_PHASE_CENTDY_fda_PHASE'], self.dic['CrossCorr_CENTDX_fda_PHASE_CENTDY_fda_PHASE'] ]
        header = 'HJD_PHASE'+'\t'+'SYSREM_FLUX3_PHASE'+'\t'+'SYSREM_FLUX3_PHASE_ERR'+'\t'+\
                 'CENTDX_fda_PHASE'+'\t'+'CENTDX_fda_PHASE_ERR'+'\t'+'CENTDY_fda_PHASE'+'\t'+'CENTDY_fda_PHASE_ERR'+'\t'+\
                 'CENTDX_fd_PHASE'+'\t'+'CENTDX_fd_PHASE_ERR'+'\t'+'CENTDY_fd_PHASE'+'\t'+'CENTDY_fd_PHASE_ERR'+'\t'+\
                 'CENTDX_f_PHASE'+'\t'+'CENTDX_f_PHASE_ERR'+'\t'+'CENTDY_f_PHASE'+'\t'+'CENTDY_f_PHASE_ERR'+'\t'+\
                 'CENTDX_PHASE'+'\t'+'CENTDX_PHASE_ERR'+'\t'+'CENTDY_PHASE'+'\t'+'CENTDY_PHASE_ERR'+'\t'+\
                 'RollCorr_SYSREM_FLUX3_PHASE_CENTDX_fda_PHASE'+'\t'+'RollCorr_SYSREM_FLUX3_PHASE_CENTDY_fda_PHASE'+'\t'+'RollCorr_CENTDX_fda_PHASE_CENTDY_fda_PHASE'+'\t'+\
                 'CrossCorr_SYSREM_FLUX3_PHASE_CENTDX_fda_PHASE'+'\t'+'CrossCorr_SYSREM_FLUX3_PHASE_CENTDY_fda_PHASE'+'\t'+'CrossCorr_CENTDX_fda_PHASE_CENTDY_fda_PHASE'
        np.savetxt(outfilename, X, delimiter='\t', header=header)
        if not self.silent:
            print 'Output saved as', outfilename
        
        #::: full time series data, binned to 10min
        outfilename = os.path.join( self.outdir, self.fieldname + '_' + self.obj_id + '_' + self.ngts_version + '_centroid_data_BIN.txt' )
        X = np.c_[ self.dic['HJD_BIN'], self.dic['SYSREM_FLUX3_BIN'], self.dic['SYSREM_FLUX3_BIN_ERR'], 
                   self.dic['CENTDX_fda_BIN'], self.dic['CENTDX_fda_BIN_ERR'], self.dic['CENTDY_fda_BIN'], self.dic['CENTDY_fda_BIN_ERR'] ]
        header = 'HJD_BIN'+'\t'+'SYSREM_FLUX3_BIN'+'\t'+'SYSREM_FLUX3_BIN_ERR'+'\t'+\
                 'CENTDX_fda_BIN'+'\t'+'CENTDX_fda_BIN_ERR'+'\t'+'CENTDY_fda_BIN'+'\t'+'CENTDY_fda_BIN_ERR'
        np.savetxt(outfilename, X, delimiter='\t', header=header)    
        if not self.silent:
            print 'Output saved as', outfilename                              
        
        #::: full time series data, every exposure, only if output is 'all'
        if self.output=='all':
            outfilename = os.path.join( self.outdir, self.fieldname + '_' + self.obj_id + '_' + self.ngts_version + '_centroid_data_ALL.txt' )
            X = np.c_[ self.dic['HJD'], self.dic['SYSREM_FLUX3'],
                       self.dic['CENTDX_fda'], self.dic['CENTDY_fda'] ]
            header = 'HJD_ALL'+'\t'+'SYSREM_FLUX3_ALL'+'\t'+ \
                     'CENTDX_fda_ALL'+'\t'+'CENTDY_fda_ALL'
            np.savetxt(outfilename, X, delimiter='\t', header=header)   
            if not self.silent:
                print 'Output saved as', outfilename
        
        
        
        
    ###########################################################################
    #::: save an info file for further external fitting
    ########################################################################### 
    def save_info(self):
        if self.outfname_info is None:
            outfilename = os.path.join( self.outdir, self.fieldname + '_' + self.obj_id + '_' + self.ngts_version + '_centroid_info.txt' )
        else:
            outfilename = self.outfname_info
            
        ind_out_phase = np.where( (self.dic['HJD_PHASE'] < -0.15) | ( (self.dic['HJD_PHASE'] > 0.15) & (self.dic['HJD_PHASE'] > 0.35) ) | (self.dic['HJD_PHASE'] > 0.65) )              
        
        header = 'FIELDNAME' + '\t' +\
                 'OBJ_ID' + '\t' +\
                 'NGTS_VERSION' + '\t' +\
                 'RA' + '\t' +\
                 'DEC' + '\t' +\
                 'CCDX_0' + '\t' +\
                 'CCDY_0' + '\t' +\
                 'FLUX_MEAN' + '\t' +\
                 'CENTDX_fda_PHASE_RMSE' + '\t' +\
                 'CENTDY_fda_PHASE_RMSE' + '\t' +\
                 'CENTDX_fd_PHASE_RMSE' + '\t' +\
                 'CENTDY_fd_PHASE_RMSE' + '\t' +\
                 'CENTDX_f_PHASE_RMSE' + '\t' +\
                 'CENTDY_f_PHASE_RMSE' + '\t' +\
                 'CENTDX_PHASE_RMSE' + '\t' +\
                 'CENTDY_PHASE_RMSE' + '\t' +\
                 'RollCorrSNR_X' + '\t' +\
                 'RollCorrSNR_Y' + '\t' +\
                 'CrossCorrSNR_X' + '\t' +\
                 'CrossCorrSNR_Y' + '\t' +\
                 'Ttest_X' + '\t' +\
                 'Ttest_Y' + '\t' +\
                 'Binom_X' + '\t' +\
                 'Binom_Y'
                 
        with open(outfilename, 'w') as f:
            f.write(header+'\n')
            f.write( self.fieldname+'\t'+\
                     self.obj_id+'\t'+\
                     self.ngts_version+'\t'+\
                     str(self.dic['RA'])+'\t'+\
                     str(self.dic['DEC'])+'\t'+\
                     str(self.dic['CCDX_0'])+'\t'+\
                     str(self.dic['CCDY_0'])+'\t'+\
                     str(self.dic['FLUX_MEAN'])+'\t'+\
                     str(np.nanstd(self.dic['CENTDX_fda_PHASE'][ind_out_phase]))+'\t'+\
                     str(np.nanstd(self.dic['CENTDY_fda_PHASE'][ind_out_phase]))+'\t'+\
                     str(np.nanstd(self.dic['CENTDX_fd_PHASE'][ind_out_phase]))+'\t'+\
                     str(np.nanstd(self.dic['CENTDY_fd_PHASE'][ind_out_phase]))+'\t'+\
                     str(np.nanstd(self.dic['CENTDX_f_PHASE'][ind_out_phase]))+'\t'+\
                     str(np.nanstd(self.dic['CENTDY_f_PHASE'][ind_out_phase]))+'\t'+\
                     str(np.nanstd(self.dic['CENTDX_PHASE'][ind_out_phase]))+'\t'+\
                     str(np.nanstd(self.dic['CENTDY_PHASE'][ind_out_phase]))+'\t'+\
                     str(self.stats['RollCorrSNR_X'])+'\t'+\
                     str(self.stats['RollCorrSNR_Y'])+'\t'+\
                     str(self.stats['CrossCorrSNR_X'])+'\t'+\
                     str(self.stats['CrossCorrSNR_Y'])+'\t'+\
                     str(self.stats['Ttest_X'])+'\t'+\
                     str(self.stats['Ttest_Y'])+'\t'+\
                     str(self.stats['Binom_X'])+'\t'+\
                     str(self.stats['Binom_Y'])+'\n' )
                         
	if not self.silent:
         print 'Output saved as', outfilename
                
 


    ###########################################################################
    #::: save an info FITS table (for pipeline use)
    ########################################################################### 
    def save_info_FITS_for_pipeline(self):
        if self.outfname_info is None:
            outfilename = os.path.join( self.outdir, self.fieldname + '_' + self.obj_id + '_' + self.ngts_version + '_centroid_info.fits' )
        else:
            outfilename = self.outfname_info
        if not outfilename.endswith('.fits'): 
            outfilename += '.fits'
            
        if os.path.exists(outfilename): os.remove(outfilename)
            
            
        ind_out_phase = np.where( (self.dic['HJD_PHASE'] < -0.15) | ( (self.dic['HJD_PHASE'] > 0.15) & (self.dic['HJD_PHASE'] > 0.35) ) | (self.dic['HJD_PHASE'] > 0.65) )              
                
                
        #The first four of these (SNR): If any SNR > 5 I would get suspicous (yellow flag). Any SNR > 7 so far always indicated a significant centroid correlation by eye (red flag).
        #The latter four (p-values): for the p-values I would get suspicous if p<0.05 (yellow flag), and see a clear trend if p<0.01 (red flag).
#        sigma = 5
#        p_min = 0.05
        
        
        header = [ 'FIELDNAME',
                 'OBJ_ID',
                 'NGTS_VERSION',
                 'RA',
                 'DEC',
                 'CCDX_0',
                 'CCDY_0',
                 'FLUX_MEAN',
                 'CENTDX_fda_PHASE_RMSE',
                 'CENTDY_fda_PHASE_RMSE',
                 'RollCorrSNR_X',
                 'RollCorrSNR_Y',
                 'CrossCorrSNR_X',
                 'CrossCorrSNR_Y',
                 'Ttest_X',
                 'Ttest_Y',
                 'Binom_X',
                 'Binom_Y',
                 'Centd_X_flag',
                 'Centd_Y_flag',
                 'Suggestion']
                                  
        data = [ self.fieldname,
                 self.obj_id,
                 self.ngts_version,
                 self.dic['RA'],
                 self.dic['DEC'],
                 self.dic['CCDX_0'],
                 self.dic['CCDY_0'],
                 self.dic['FLUX_MEAN'],
                 np.nanstd(self.dic['CENTDX_fda_PHASE'][ind_out_phase]),
                 np.nanstd(self.dic['CENTDY_fda_PHASE'][ind_out_phase]),
                 self.stats['RollCorrSNR_X'],
                 self.stats['RollCorrSNR_Y'],
                 self.stats['CrossCorrSNR_X'],
                 self.stats['CrossCorrSNR_Y'],
                 self.stats['Ttest_X'],
                 self.stats['Ttest_Y'],
                 self.stats['Binom_X'],
                 self.stats['Binom_Y'],
                 int(self.centd_flag['X']),
                 int(self.centd_flag['Y']),
                 self.suggestion ]
                 
        formats = ['11A',
                   '6A',
                   '20A',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1E',
                   '1I',
                   '1I',
                   '80A']
        
        columns = []
        for i,_ in enumerate(data):            
            c = fits.Column(name=header[i], array=[data[i]], format=formats[i])
            columns.append(c)
            
        t = fits.BinTableHDU.from_columns(columns)
        t.writeto(outfilename)
                         
	if not self.silent:
         print 'Output saved as', outfilename


               
                
    ###########################################################################
    #::: save an info file for further external fitting
    ########################################################################### 
    def save_flagfile(self):
        if self.flagfile is not None:
            
            with open(self.flagfile, 'a') as f:
                
                if self.valid:
                    #TODO: ind_out_phase is very rough!
                    ind_out_phase = np.where( (self.dic['HJD_PHASE'] < -0.15) | ( (self.dic['HJD_PHASE'] > 0.15) & (self.dic['HJD_PHASE'] > 0.35) ) | (self.dic['HJD_PHASE'] > 0.65) )
                    f.write( self.fieldname+'\t'+\
                            self.obj_id+'\t'+\
                            self.ngts_version+'\t'+\
                            str(self.dic['RA'])+'\t'+\
                            str(self.dic['DEC'])+'\t'+\
                            str(self.dic['CCDX_0'])+'\t'+\
                            str(self.dic['CCDY_0'])+'\t'+\
                            str(self.dic['FLUX_MEAN'])+'\t'+\
                            str(np.nanstd(self.dic['CENTDX_fda_PHASE'][ind_out_phase]))+'\t'+\
                            str(np.nanstd(self.dic['CENTDY_fda_PHASE'][ind_out_phase]))+'\t'+\
                            str(np.nanstd(self.dic['CENTDX_fd_PHASE'][ind_out_phase]))+'\t'+\
                            str(np.nanstd(self.dic['CENTDY_fd_PHASE'][ind_out_phase]))+'\t'+\
                            str(np.nanstd(self.dic['CENTDX_f_PHASE'][ind_out_phase]))+'\t'+\
                            str(np.nanstd(self.dic['CENTDY_f_PHASE'][ind_out_phase]))+'\t'+\
                            str(np.nanstd(self.dic['CENTDX_PHASE'][ind_out_phase]))+'\t'+\
                            str(np.nanstd(self.dic['CENTDY_PHASE'][ind_out_phase]))+'\t'+\
                            str(self.stats['RollCorrSNR_X'])+'\t'+\
                            str(self.stats['RollCorrSNR_Y'])+'\t'+\
                            str(self.stats['CrossCorrSNR_X'])+'\t'+\
                            str(self.stats['CrossCorrSNR_Y'])+'\t'+\
                            str(self.stats['Ttest_X'])+'\t'+\
                            str(self.stats['Ttest_Y'])+'\t'+\
                            str(self.stats['Binom_X'])+'\t'+\
                            str(self.stats['Binom_Y'])+'\n' )
                            
                else:
                    f.write( '#'+self.fieldname+'\t'+\
                            self.obj_id+'\t'+\
                            self.flagfile_text+'\n' )
 
            print 'flagfile updated.'
            
#        else:
#            print "Note: flagfile is 'None'."
            
    
    
    def detrend(self):
        detrender = detrend_centroid_external.detrend_centroid(self.dic, self.dic_nb, method=self.method, R_min=self.R_min, N_top_max=self.N_top_max, dt=self.dt)
        self.dic, self.dic_nb = detrender.run()   
        
        
        
    def check_object(self):
        self.valid = True
        
        if self.nancut is None:
            for key in ['X','Y']:
                N_nan = np.count_nonzero(np.isnan(self.dic['CENTD'+key]))
                N_tot = len(self.dic['CENTD'+key])
                #if more than half of all entries are NaN, declare the object as invalid
                if (1.*N_nan/N_tot > 0.5): self.valid = False
        else:
            N_nan = np.count_nonzero(self.nancut)
            N_tot = len( self.nancut )
            #if more than half of all entries are NaN, declare the object as invalid
            if (1.*N_nan/N_tot > 0.5): self.valid = False

        if self.valid == False:
            warnings.warn('Object '+self.obj_id+' skipped: too many NaNs in CENTD array (' + str(100.*N_nan/N_tot)[:4] + ' %).')
            self.flagfile_text = '(skipped: too many NaNs in CENTD array, ' + str(100.*N_nan/N_tot)[:4] + ' %)'
            
            
    ###########################################################################
    #::: run (for individual targets)
    ###########################################################################    
    def run(self):
        
        print self.fieldname, self.obj_id, self.ngts_version, self.source
        
        #::: to load data
        self.load_object()
        if not self.silent: print 'loaded object.'
        
        self.check_object()
        
        if self.valid:
            if not self.silent: print 'object is valid.'
            #::: overwrite transit parameters with a simulated transit
    #        self.dic = simulate_signal.simulate( self.dic, tipo='EB_FGKM', plot=False )
            
            #::: load reference stars
            self.load_neighbours()
            if not self.silent: print 'loaded neighbours.'
            
            #::: load crossmatching information
            self.load_catalog()
            if not self.silent: print 'loaded catalog.'
            
            self.mark_eclipses()
            if not self.silent: print 'marked eclipses and out-of-eclipse.'
            
            #::: assign colorcode corresponding to airmass
            if self.do_plot == 'all':
                self.assign_airmass_colorcode()
                if not self.silent: print 'assigned airmass color codes.'
            
            #::: detrend the centroid
            self.detrend()
            if not self.silent: print 'detrended externally.'
            
            #::: bin the dictionary (_BIN and _BIN_ERR)
            self.binning()
            if not self.silent: print 'binned.'
    
            #TODO shift the plot parts of the following functions into seperate script
    
            #::: to study a target in detail
            self.phase_fold()
            if not self.silent: print 'phase folded.'
            
            #::: calculate and plot ccf and acf
            self.cross_correlate()
            if not self.silent: print 'cross-correlation calculated.'
            
            #::: calculate stats (SNR, Ttest, Bimod-test)
            self.do_stats()
            if not self.silent: print 'stats calculated.'
            
            #::: make a suggestion as to which object eclipses
            self.make_suggestion()
            
            #plots
            if self.do_plot == 'all':
                analyse_neighbours.plot( self.dic, self.dic_nb, self.outdir, self.fieldname, self.obj_id, self.ngts_version, dt=self.dt )


            if self.do_plot in ('minimal','all'):
                inspect_blends.plot( self )
                
                self.plot_scatter_matrix()
            
                self.plot_phase_folded_curves()
            
    ##        self.plot_rainplot_summary() #exclude
    ##        
    ##        self.plot_rainplot_per_night() #exclude
    ##        
    ##        self.plot_detrending_over_time() #exclude
    #        
    #        self.plot_detrending() #exclude
            
                self.plot_info_page()
            
                self.plot_stacked_image()
            
                self.save_pdf()
                if not self.silent: print 'plots saved.'
                
                
            if self.do_plot == 'pipeline':
                self.plot_for_pipeline()
                self.save_png()
                if not self.silent: print 'plots saved.'
                
                
            
            #data files
            if self.output=='pipeline':
                self.save_info_FITS_for_pipeline()
                
            else:
                self.save_data()
                if not self.silent: print 'data files saved.'
            
                self.save_info()
                if not self.silent: print 'info files saved.'
          
          
          
            #::: if silent, only give the outdir
            if self.silent & (self.output!='pipeline'):
                print 'Output saved in', self.outdir
                
        #if not valid, pass
        else:
            pass
    
                      
        self.save_flagfile()
        
        
        
if __name__ == '__main__':
    pass

#    C = centroid('NG0613-3633', '008997', 'CYCLE1706', do_plot='pipeline', output='pipeline') 
#    C = centroid('NG0613-3633', '010712', 'TEST18', do_plot='pipeline', output='pipeline') 
#    C.run()




# -*- coding: utf-8 -*-
"""
Created on Tue Oct 18 11:27:46 2016

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""

import warnings
import numpy as np
import matplotlib.pyplot as plt
from astropy.modeling import models, fitting
from matplotlib.colors import LogNorm
import os, sys, socket
import glob
import fitsio
from photutils.background import Background
from photutils import daofind
from astropy.stats import sigma_clipped_stats
from photutils import CircularAperture


from ngtsio import ngtsio


def standard_fnames(fieldname, root=None):
    
    if (root is None):
        
        #::: on laptop (OS X)
        if sys.platform == "darwin":
            root = '/Users/mx/Big_Data/BIG_DATA_NGTS/2016/STACKED_IMAGES/'
                
        #::: on Cambridge servers
        elif 'ra.phy.cam.ac.uk' in socket.gethostname():
            root = '/appch/data/mg719/ngts_pipeline_output/STACKED_IMAGES/'
                              
    filename = glob.glob( os.path.join( root, 'DITHERED_STACK_'+fieldname+'*.fits' ) )[0]
                        
    return filename
        
        
        
def load(fieldname):
    
    filename = standard_fnames(fieldname)
    data = fitsio.read( filename )
    
    return data
    
    

def plot(fieldname, x, y, r=15, ax=None, show_apt=True):
    '''
        Note: it is important to transform the coordinates!
        
        resolution of stacked images: (8192, 8192)
        resolution of normal CCD images: (2048, 2048)
        -> scaling factor of 4
        
        x and y must additionally be shifted by - 0.5 + 0.5/scale 
        due to miss-match of different image origins in fits vs python vs C
        
        y axis is originally flipped if image is laoded via fitsio.read
    '''
    scale = 4
    x = x - 0.5 + 0.5/scale
    y = y - 0.5 + 0.5/scale
    
    stacked_image = load(fieldname)
#    bkg = np.nanmedian(stacked_image)
#    stacked_image -= bkg
    
    x0 = np.int( x - r )*scale #*4 as stacked images have 4 x higher resolution than normal images
    x1 = np.int( x + r )*scale
    y0 = np.int( y - r )*scale
    y1 = np.int( y + r )*scale
    
    stamp = stacked_image[ y0:y1, x0:x1 ] #x and y are switched in indexing
    bkg = Background(stamp, (50,50), filter_shape=(3, 3), method='median')
    stamp -= bkg.background
    
    if ax is None:
        fig, ax = plt.subplots(figsize=(6,6))
        
    im = ax.imshow( stamp, norm=LogNorm(vmin=1, vmax=1000), interpolation='nearest', origin='lower', extent=1.*np.array([x0,x1,y0,y1])/scale )
    plt.colorbar(im)
    
    if show_apt == True:
        ax.scatter( x, y, c='r' )
        circle1 = plt.Circle((x, y), 3, color='r', fill=False, linewidth=3)
        ax.add_artist(circle1)

    fit_Gaussian2D(stamp)#, x, y, x0, x1, y0, y1)
    
    
    

def remove_bkg(z):
    bkg = Background(z, (50,50), filter_shape=(3, 3), method='median')
    
    plt.figure()
    im = plt.imshow(z, norm=LogNorm(vmin=1, vmax=1000), origin='lower', cmap='Greys')
    plt.colorbar(im)
    
    plt.figure()
    im = plt.imshow(bkg.background, origin='lower', cmap='Greys')
    plt.colorbar(im)
    
    plt.figure()
    im = plt.imshow(z - bkg.background, norm=LogNorm(vmin=1, vmax=1000), interpolation='nearest', origin='lower')
    plt.colorbar(im)
           
           
           

def fit_Gaussian2D(z):
    
#    xx, yy = np.meshgrid( np.arange(x0, x1), np.arange(y0, y1) )

    x = np.arange(0,z.shape[0])
    y = np.arange(0,z.shape[1])
    xx, yy = np.meshgrid( x,y )

    plt.figure()
    im = plt.imshow(z, norm=LogNorm(vmin=1, vmax=1000), origin='lower', cmap='Greys')
    plt.colorbar(im)
    
    
    #::: PHOTUTILS SOURCE DETECTION
    data = z
    mean, median, std = sigma_clipped_stats(data, sigma=3.0, iters=5)     
    sources = daofind(data - median, fwhm=4.0, threshold=4.*std)
    positions = (sources['xcentroid'], sources['ycentroid'])
    apertures = CircularAperture(positions, r=3.*4)
#    plt.figure()
#    plt.imshow(data, cmap='Greys', norm=LogNorm(vmin=1, vmax=1000))
    apertures.plot(color='blue', lw=1.5, alpha=0.5)
    #:::
    
    
    amplitude_A = np.max( z )
    x_A_mean = x[-1]/2
    y_A_mean = y[-1]/2
    x_A_stddev = 4. #assume FWHM of 1 pixel
    y_A_stddev = 4.
    
    amplitude_B = 0. #assume no second peak
    x_B_mean = x[-1]/2
    y_B_mean = y[-1]/2
    x_B_stddev = 4. #assume FWHM of 1 pixel
    y_B_stddev = 4.
    
    
    # Now to fit the data create a new superposition with initial
    # guesses for the parameters:
    gg_init = models.Gaussian2D(amplitude_A, x_A_mean, y_A_mean, x_A_stddev, y_A_stddev) \
                + models.Gaussian2D(amplitude_B, x_B_mean, y_B_mean, x_B_stddev, y_B_stddev)
    fitter = fitting.SLSQPLSQFitter()
    gg_fit = fitter(gg_init, xx, yy, z)
    
    print gg_fit
    print gg_fit.parameters
    print gg_fit(xx, yy)
    print gg_fit(0,0)
    print gg_fit(60,60)
    
    
    plt.figure()
    plt.imshow(gg_fit(xx,yy), label='Gaussian')


    
        
        
def test_plot(fieldname, x, y, r=15, ax=None):
    '''
        Note: it is important to transform the coordinates!
        
        resolution of stacked images: (8192, 8192)
        resolution of normal CCD images: (2048, 2048)
        
        y axis is originally flipped if image is laoded via fitsio.read
    '''
    scale = 4
    
    stacked_image = load(fieldname)
    stacked_image -= 0.95*np.nanmedian(stacked_image) #200
    
#    fig, ax1 = plt.subplots()
#    ax1.hist( stacked_image.flatten(), bins=np.linspace(200, 240, 50))
    
    x0 = np.int( x - r )*scale #*4 as stacked images have 4 x higher resolution than normal images
    x1 = np.int( x + r )*scale
    y0 = np.int( y - r )*scale
    y1 = np.int( y + r )*scale
    
    im = stacked_image[ y0:y1, x0:x1 ]
    
    if ax is None:
        fig, axes = plt.subplots(1,3,figsize=(18,6))
    
    for i in range(3):
        if i == 0:
            x = x
            y = y
            title = 'no shift'
        elif i == 1:
            x = x - 0.5 + 0.5/scale
            y = y - 0.5 + 0.5/scale
            title = 'shift - 3/4 * 0.5'
        elif i == 2:
            x = x -0.5
            y = y -0.5
            title = 'shift - 0.5'
        ax = axes[i]
        
        ax.imshow( im, norm=LogNorm(vmin=1, vmax=1000), interpolation='nearest', origin='lower', extent=1.*np.array([x0,x1,y0,y1])/scale )
        ax.scatter( x, y, c='r' )
        circle1 = plt.Circle((x, y), 3, color='r', fill=False, linewidth=3)
        ax.add_artist(circle1)
        ax.set_title(title)
        
        
        
if __name__ == '__main__':
    fieldname = 'NG0304-1115'
#    obj_id = 9861
#    obj_id = 992
    obj_id = 1703
#    obj_id = 6190
#    obj_id = 24503
#    obj_id = 7505
#    obj_id = 16335
    
    data = ngtsio.get(fieldname, ['CCDX','CCDY'], obj_id=obj_id)
    
    plot(fieldname, np.mean(data['CCDX']), np.mean(data['CCDY']), r=15) #obj 9861
#    plot('NG0304-1115', 1112, 75, r=15)
    
    
